var UserPreferences = (function () {
    function UserPreferences() {
        this.userName = "";
        this.userId = "";
        this.password = "";
        this.pincode = "";
        this.refreshMinutes = 15;
        this.savePassword = false;
        ko.track(this);
    }
    return UserPreferences;
})();
var TenantData = (function () {
    function TenantData() {
        this.mcMessages = [];
        this.selectedIncident = null;
        this.selectedUser = null;
        this.selectedSR = null;
        this.selectedMessage = null;
        this.clear();
        ko.track(this);
    }
    TenantData.prototype.clear = function () {
        this.isAOBO = false;
        this.companyInfo = null;
        this.domains = [];
        this.skus = [];
        this.lastEventRefreshMilliSec = 0;
        this.filteredUsers = [];
        this.userFilter = "";
        this.recentUsers = [];
        this.filteredSRs = [];
        this.recentSRs = [];
        this.incidents = [];
        this.services = [];
    };
    return TenantData;
})();
var UserData = (function () {
    function UserData() {
        this.cookie = null;
        this.cookieExpired = false;
        this.lastCookieRefreshMilliSec = 0;
        this.lastVisitedPage = "";
        this.lastPrimaryPivot = "";
        this.lastSecondPivot = "";
        this.userRole = 0 /* None */;
        this.partnerRole = 1 /* FullAdministration */;
        this.locale = "en-us";
        this.flaggedCompanies = [];
        this.recentCompanies = [];
        this.filteredCompanies = [];
        this.companyFilter = "";
        this.affectedCompanyIds = [];
        this.clientIncidents = [];
        this.selectedIncidentId = "";
        ko.track(this);
    }
    UserData.prototype.clearClientList = function () {
        this.companyFilter = "";
        this.filteredCompanies.removeAll();
        this.selectedIncidentId = "";
    };
    return UserData;
})();
var Cache = (function () {
    function Cache() {
        this.preferences = new UserPreferences();
        this.userData = new UserData();
        this.savedBy = "";
        this.version = "";
        this.uiVersion = Config.uiVersion;
        this.appVersion = "";
        this.nativeSwipe = false;
        this.nativeBack = false;
        this.isNetworkAvailable = true;
        assert(!Cache._instance);
        Cache._instance = this;
        Cache._tenantData = new TenantData();
        if (isDebug() && Config.allowPIISavedForDebug)
            this._tenantDataDebug = this.tenantData;
        this.smoke();
        ko.track(this);
        var data = this.load();
        if (data) {
            if (!data.uiVersion || parseInt(data.uiVersion.split('.')[2]) < Config.minVersion) {
                console.log("out-of-date cache:" + localStorage.getItem(Cache.key));
                data = null;
            }
        }
        var p = this.preferences;
        if (data) {
            this.merge(this, data);
        }
        assert(p == this.preferences);
    }
    Object.defineProperty(Cache.prototype, "tenantData", {
        get: function () {
            return Cache._tenantData;
        },
        enumerable: true,
        configurable: true
    });
    Cache.getInstance = function () {
        if (!Cache._instance)
            Cache._instance = new Cache();
        return Cache._instance;
    };
    Cache.prototype.merge = function (original, extended) {
        assert(typeof original === 'object');
        for (var key in extended) {
            if (!Array.isArray(extended[key]) && typeof extended[key] === 'object' && original[key]) {
                assert(typeof original[key] === 'object');
                this.merge(original[key], extended[key]);
            }
            else
                original[key] = extended[key];
        }
        return original;
    };
    Cache.prototype.clearUserDataAndPreferences = function () {
        this.userData = new UserData();
        this.preferences = new UserPreferences();
    };
    Cache.prototype.smoke = function () {
        Trace.logInfo("test cache");
        if (!localStorage) {
            alert("localStorage not supported in this browser");
            return;
        }
        try {
            localStorage.setItem("test", "test");
            var value = localStorage.getItem("test");
            localStorage.removeItem("test");
        }
        catch (e) {
        }
        if (value != "test")
            alert("cache is disabled in this browser. Feature will not work");
    };
    Cache.prototype.getPageName = function () {
        return window.location.pathname.substring(location.pathname.lastIndexOf("/") + 1);
    };
    Cache.prototype.save = function () {
        localStorage.setItem(Cache.key, this.toJson());
        Trace.logVerbose("Cache is saved");
    };
    Cache.prototype.toJson = function () {
        this.userData.lastVisitedPage = this.getPageName();
        this.version = NativeHelper.getAppVersion();
        var json = JSON.stringify(this);
        return json;
    };
    Cache.prototype.load = function () {
        var json = localStorage.getItem(Cache.key);
        Trace.logVerbose("Load() " + json);
        try {
            return JSON.parse(json);
        }
        catch (e) {
            console.log(e);
        }
        return null;
    };
    Cache.key = "cache";
    Cache._instance = null;
    return Cache;
})();
var NativeHelper = (function () {
    function NativeHelper() {
    }
    NativeHelper.prototype.exitApp = function () {
    };
    NativeHelper.isNetworkAvailable = function () {
        return Cache.getInstance().isNetworkAvailable;
    };
    NativeHelper.isWiFiOn = function () {
        return true;
    };
    NativeHelper.getAppVersion = function () {
        return Cache.getInstance().appVersion;
    };
    NativeHelper.getUserLocale = function () {
        return "en-us";
    };
    return NativeHelper;
})();
if (window["DeviceNativeHelper"])
    window["NativeHelper"] = window["DeviceNativeHelper"];
function setNative(json) {
    var cache = Cache.getInstance();
    var parsed = JSON.parse(json);
    for (var name in parsed) {
        assert(name in cache);
        var value = parsed[name];
        cache[name] = value;
    }
    cache.save();
    return json;
}
;
//# sourceMappingURL=Cache.js.map