var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var ClientsView = (function (_super) {
    __extends(ClientsView, _super);
    function ClientsView() {
        _super.call(this);
        this.canRefresh = false;
        this.filteredItems = [];
        this.showToolbar = true;
        this.bind();
        this.userData.affectedCompanyIds = [];
        this.asyncGetPartnerFlaggedList();
    }
    Object.defineProperty(ClientsView.prototype, "searchPlaceHolder", {
        get: function () {
            return "Search client by domain or name";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ClientsView.prototype, "searchString", {
        get: function () {
            return this.userData.companyFilter;
        },
        set: function (filter) {
            this.userData.companyFilter = filter;
        },
        enumerable: true,
        configurable: true
    });
    ClientsView.prototype.onReload = function () {
        if (!this.userData.filteredCompanies.length && !this.searchString)
            this.asyncSearchCompanyInfo("", false);
        this.setSearchString(this.searchString);
    };
    ClientsView.prototype.isPrimary = function () {
        return !this.userData.selectedIncidentId;
    };
    ClientsView.prototype.getTitle = function () {
        if (this.userData.selectedIncidentId)
            return "Affected by " + this.userData.selectedIncidentId;
        else
            return _super.prototype.getTitle.call(this);
    };
    ClientsView.prototype.onSearchClearClick = function () {
        _super.prototype.onSearchClearClick.call(this);
        this.userData.selectedIncidentId = "";
    };
    ClientsView.prototype.getFlagClass = function (domainInfo) {
        var isFlagged = false;
        for (var j = 0; j < this.userData.flaggedCompanies.length; j++) {
            if (this.userData.flaggedCompanies[j].TenantId == domainInfo.TenantId) {
                isFlagged = true;
                break;
            }
        }
        return this.getIconClass(isFlagged ? "flag" : "unflag");
    };
    ClientsView.prototype.getRightToolbarText = function () {
        return this.userData.flaggedCompanies.length ? "Flagged" : "Recent";
    };
    ClientsView.prototype.toggleToolbar = function (isRight) {
        this.rightToolbarSelected = isRight;
    };
    ClientsView.prototype.getItems = function () {
        if (this.rightToolbarSelected) {
            var domains = this.userData.flaggedCompanies.length ? this.userData.flaggedCompanies : this.userData.recentCompanies;
            if (this.userData.selectedIncidentId) {
                var affected = this.userData.filteredCompanies.map(function (c) {
                    return c.TenantId;
                });
                domains = this._getAffectedTenants(domains, affected);
            }
            else
                this._joinAffectedTenants(domains, this.userData.affectedCompanyIds);
        }
        else {
            domains = this.userData.filteredCompanies;
        }
        return domains;
    };
    ClientsView.prototype.search = function (searchString, blockUI) {
        if (this.hasInvalidCharacters(searchString)) {
            Trace.showErrorMessage(Strings.InvalidSearchString);
            return;
        }
        this.asyncSearchCompanyInfo(searchString, blockUI);
        if (blockUI)
            this.logEvent("SearchClients");
    };
    ClientsView.prototype.hasInvalidCharacters = function (s) {
        var invalid = "(['\"_,*+?^${}()|[]/\\])".split("");
        for (var i = 0; i < invalid.length; i++) {
            if (s.indexOf(invalid[i]) > -1)
                return true;
        }
        return false;
    };
    ClientsView.prototype.clickItem = function (company) {
        this.asyncGetCompanyInfo(company);
        this.logEvent("Client");
    };
    ClientsView.prototype.asyncGetCompanyInfo = function (company) {
        if (!this.isPartner())
            return;
        var serviceParams = {
            lastCookie: this.userData.cookie,
            tenantId: company.TenantId
        };
        var _this = this;
        var onSuccess = function (companyInfo) {
            if (companyInfo) {
                if (!_this.tenantData.companyInfo || _this.tenantData.companyInfo.ObjectId != company.TenantId) {
                    _this.addToRecent(company);
                    _this.tenantData.clear();
                    _this.tenantData.companyInfo = companyInfo;
                    _this.tenantData.isAOBO = true;
                }
                _this.goTo(HealthView);
            }
            else {
                Trace.showErrorMessage(Strings.ClientsNotFound);
            }
        };
        var onError;
        this.asyncRequest(Config.serviceURL, Config.getCompanyInfoTag, serviceParams, onSuccess.bind(this), null, true);
    };
    ClientsView.prototype.onAjaxError = function (statusCode, statusDescription, error, api, blockUI) {
        if (api == Config.getCompanyInfoTag && statusCode == 401) {
            Trace.showErrorMessage(Strings.ClientNotFound);
            return true;
        }
        else {
            return _super.prototype.onAjaxError.call(this, statusCode, statusDescription, error, api, blockUI);
        }
    };
    ClientsView.prototype.asyncSearchCompanyInfo = function (searchString, blockUI) {
        if (blockUI === void 0) { blockUI = false; }
        if (!this.isPartner())
            return;
        var serviceParams = {
            lastCookie: this.userData.cookie,
            searchString: searchString,
            filter: searchString,
            resultSize: Config.resultSize
        };
        var _this = this;
        var onSuccess = function (companies) {
            if (companies && companies.length) {
                _this.searchString = searchString;
                _this.userData.selectedIncidentId = "";
                _this.userData.filteredCompanies = companies;
                if (blockUI)
                    _this.rightToolbarSelected = false;
            }
            else {
                Trace.showErrorMessage(Strings.ClientsNotFound);
            }
        };
        this.asyncRequest(Config.serviceURL, Config.searchCompanyInfoTag, serviceParams, onSuccess.bind(this), null, blockUI);
    };
    ClientsView.prototype.asyncGetPartnerFlaggedList = function () {
        if (!this.isPartner())
            return;
        var serviceParams = {
            lastCookie: this.userData.cookie,
        };
        this.asyncRequest(Config.serviceURL, Config.getPartnerFlaggedList, serviceParams, this.onFlaggedListReturned.bind(this));
    };
    ClientsView.prototype.asyncFindAffectedTenants = function (companyIds) {
        assert(companyIds && companyIds.length);
        if (!this.isPartner())
            return;
        var serviceParams = {
            lastCookie: this.userData.cookie,
            companyIds: companyIds
        };
        this.asyncRequest(Config.serviceURL, Config.findAffectedTenantsTag, serviceParams, this.onAffectedTenantsReturned.bind(this));
    };
    ClientsView.prototype.addToRecent = function (client) {
        for (var j = 0; j < this.userData.recentCompanies.length; j++) {
            if (this.userData.recentCompanies[j].TenantId == client.TenantId) {
                this.userData.recentCompanies.splice(j, 1);
                break;
            }
        }
        client.Affected = false;
        this.userData.recentCompanies.unshift(client);
    };
    ClientsView.prototype.onFlaggedListReturned = function (response) {
        if (response)
            this.userData.flaggedCompanies = response;
        if (response && response.length) {
            if (!this.userData.flaggedCompanies.length && !this.userData.filteredCompanies.length) {
                this.rightToolbarSelected = true;
            }
            if (!this.userData.affectedCompanyIds.length || response.length != this.userData.flaggedCompanies.length) {
                var companyIds = response.map(function (domain) {
                    return domain.TenantId;
                });
                this.asyncFindAffectedTenants(companyIds);
            }
        }
    };
    ClientsView.prototype.onAffectedTenantsReturned = function (response) {
        if (response) {
            this.userData.affectedCompanyIds = response;
        }
    };
    ClientsView.prototype._joinAffectedTenants = function (clients, affected) {
        for (var i = 0; i < clients.length; i++) {
            clients[i].Affected = (affected.indexOf(clients[i].TenantId) >= 0);
            ko.track(clients[i]);
        }
    };
    ClientsView.prototype._getAffectedTenants = function (clients, affected) {
        var result = [];
        for (var i = 0; i < clients.length; i++) {
            if (affected.indexOf(clients[i].TenantId) >= 0) {
                clients[i].Affected = false;
                result.push(clients[i]);
            }
        }
        return result;
    };
    ClientsView.menuName = "Clients";
    ClientsView.incidentPrefix = "Affected by ";
    return ClientsView;
})(ViewModelBase);
//# sourceMappingURL=ClientsView.js.map