var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var HealthView = (function (_super) {
    __extends(HealthView, _super);
    function HealthView() {
        _super.call(this);
        this.showToolbar = true;
        this.bind();
    }
    HealthView.prototype.onReload = function () {
        this.rightToolbarSelected = this.isPartnerNotAOBO();
    };
    HealthView.prototype.isPrimary = function () {
        return true;
    };
    HealthView.prototype.getLeftToolbarText = function () {
        if (this.isPartnerNotAOBO())
            return "Me";
        else
            return "Services";
    };
    HealthView.prototype.getRightToolbarText = function () {
        if (this.isPartnerNotAOBO())
            return "Clients";
        else
            return "Incidents";
    };
    HealthView.getAffectedCountText = function (affectedTenantCount) {
        var affected = "";
        if (affectedTenantCount > 0)
            affected = "Affected clients: " + affectedTenantCount;
        else if (affectedTenantCount === 0)
            affected = "Global incident";
        return affected;
    };
    HealthView.prototype.getServiceIncidents = function () {
        var results = [];
        if (!this.rightToolbarSelected) {
            var services = this.tenantData.services;
            var events = this.tenantData.incidents;
            if (services && events) {
                for (var l = 0; l < services.length; l++) {
                    results.push(this._joinServiceInfoWithIncident(services[l], events));
                }
            }
            return results;
        }
        else {
            if (this.isPartnerNotAOBO())
                events = this.userData.clientIncidents;
            else
                events = this.tenantData.incidents;
            results = events.sort(HealthView.compareServerity);
        }
        return results;
    };
    HealthView.prototype.getEmptyListString = function () {
        if (!this.rightToolbarSelected)
            return "Couldn't find any subscribed service.";
        else
            return "Couldn't find any incidents.";
    };
    HealthView.compareServerity = function (a, b) {
        var diff = a.status - b.status;
        if (diff)
            return diff;
        else
            return a.serviceName > b.serviceName ? 1 : -1;
    };
    HealthView.prototype.clickServiceIncident = function (incident) {
        if (incident.status == 99)
            return;
        this.tenantData.selectedIncident = incident;
        if (incident.incidentId)
            if (this.isPartnerNotAOBO())
                this.logEvent("ClientsIncident");
            else
                this.logEvent("Incident");
        else
            this.logEvent("Service");
        this.goTo(IncidentsView);
    };
    HealthView.getStatus = function (status) {
        switch (status.toLowerCase()) {
            case "service interruption":
                return 0 /* ServiceInterruption */;
            case "service degradation":
                return 1 /* ServiceDegradation */;
            case "restoring service":
                return 2 /* RestoringService */;
            case "extended recovery":
                return 3 /* ExtendedRecovery */;
            case "investigating":
                return 4 /* Investigating */;
            case "service restored":
                return 5 /* ServiceRestored */;
            case "false positive":
                return 6 /* FalsePositive */;
            case "post-incident report published":
                return 7 /* PIRPublished */;
            case "information unavailable":
                return 8 /* InformationUnavailable */;
            case "service operational":
                return 99 /* ServiceOperational */;
            default:
                return 100 /* Unknown */;
        }
    };
    HealthView.getStatusColor = function (status) {
        assert(status >= 0);
        var color;
        switch (status) {
            case 99 /* ServiceOperational */:
            case 6 /* FalsePositive */:
            case 5 /* ServiceRestored */:
            case 7 /* PIRPublished */:
                color = "#00FF00";
                break;
            case 0 /* ServiceInterruption */:
            case 1 /* ServiceDegradation */:
            case 2 /* RestoringService */:
            case 3 /* ExtendedRecovery */:
            case 4 /* Investigating */:
                color = "yellow";
                break;
            case 8 /* InformationUnavailable */:
                color = "grey";
                break;
            case 100 /* Unknown */:
            default:
                color = "white";
                break;
        }
        return "8px solid " + color;
    };
    HealthView.toServiceIncident = function (event) {
        var messages = event.Messages.sort(function (m1, m2) {
            return ViewModelBase.getDate(m2.PublishedTime).getTime() - ViewModelBase.getDate(m1.PublishedTime).getTime();
        });
        return {
            serviceName: event.AffectedServiceHealthStatus[0].ServiceName,
            featureName: event.AffectedServiceHealthStatus[0].ServiceFeatureStatus[0].FeatureName,
            incidentId: event.Id,
            status: HealthView.getStatus(event.Status),
            statusString: event.Status,
            affectedTenantCount: event.AffectedTenantCount,
            messages: messages
        };
    };
    HealthView.prototype._joinServiceInfoWithIncident = function (serviceInfo, incidents) {
        var incident;
        for (var i = 0; i < incidents.length; i++) {
            if (incidents[i].serviceName == serviceInfo.ServiceName && (!incident || incidents[i].status < incident.status)) {
                incident = incidents[i];
            }
        }
        if (incident) {
            return incident;
        }
        else
            return {
                serviceName: serviceInfo.ServiceName,
                featureName: "",
                incidentId: "",
                status: !this.tenantData.lastEventRefreshMilliSec ? 100 : 99,
                statusString: "",
                affectedTenantCount: -1,
                messages: null
            };
    };
    HealthView.menuName = "Health";
    return HealthView;
})(ViewModelBase);
var IncidentsView = (function (_super) {
    __extends(IncidentsView, _super);
    function IncidentsView() {
        _super.call(this);
        this.canRefresh = false;
        this.bind();
    }
    IncidentsView.prototype.onReload = function () {
        this.incidents = this.getIncidents();
    };
    IncidentsView.prototype.singleIncident = function () {
        return this.incidents.length == 1;
    };
    IncidentsView.prototype.getTitle = function () {
        if (this.singleIncident())
            return this.incidents[0].incidentId;
        else
            return _super.prototype.getTitle.call(this);
    };
    IncidentsView.prototype.getSecondaryTitle = function () {
        if (this.singleIncident() && this.incidents[0].affectedTenantCount >= 0)
            return this.incidents[0].serviceName;
        else
            return "";
    };
    IncidentsView.prototype.clickIncident = function (incident) {
        if (incident.affectedTenantCount > 0) {
            this.logEvent("AffectedTenants");
            this.asyncFindAffectedTenantsForPartner(incident);
        }
    };
    IncidentsView.prototype.asyncFindAffectedTenantsForPartner = function (incident) {
        var client = this.tenantData.isAOBO;
        var preCheckTenants = this.userData.flaggedCompanies.map(function (domain) {
            return domain.TenantId;
        });
        var serviceParams = {
            lastCookie: this.userData.cookie,
            preCheckTenants: preCheckTenants,
            incidentId: this.tenantData.selectedIncident.incidentId
        };
        var _this = this;
        var callback = function (response) {
            _this.userData.clearClientList();
            _this.userData.selectedIncidentId = incident.incidentId;
            _this.userData.filteredCompanies = response;
            this.goTo(ClientsView);
        };
        this.asyncRequest(Config.serviceURL, Config.findAffectedTenantsForPartnerTag, serviceParams, callback, null, true);
    };
    IncidentsView.prototype.getIncidentTitle = function (incident) {
        if (this.singleIncident())
            return this.viewAffectedText(incident.affectedTenantCount) || incident.serviceName;
        else
            return incident.incidentId;
    };
    IncidentsView.prototype.viewAffectedText = function (affectedTenantCount) {
        if (affectedTenantCount > 0)
            return "View " + affectedTenantCount + " affected client" + (affectedTenantCount > 1 ? "s" : "");
        else if (affectedTenantCount === 0)
            return "Global incident";
        else
            return "";
    };
    IncidentsView.prototype.getIncidents = function () {
        var result = [];
        var serviceName = this.tenantData.selectedIncident.serviceName;
        if (this.getInstance(HealthView).rightToolbarSelected)
            result = [this.tenantData.selectedIncident];
        else {
            result = this.tenantData.incidents.filter(function (incident) {
                return incident.serviceName == serviceName;
            });
        }
        return result.sort(HealthView.compareServerity);
    };
    return IncidentsView;
})(ViewModelBase);
//# sourceMappingURL=HealthView.js.map