/// <reference path="Utilities.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="ViewModelBase.ts" />

class LoginView extends ViewModelBase {
	private static _clickCount = 0;

	constructor(isRoot) {
		super(isRoot);

		/*
		if (this.userData.cookie && this.getCurrentRefreshMinutes() < 24 * 60) {
			this.navigateTo("PivotView.html");
			return;
		}*/

		this.bind();

		$("#username").val(this.preferences.userName || $("#username").val());
		if (this.preferences.password) {
			$("#password").val(this.preferences.password);
		}

		if (this.userData.cookieExpired) {
			Trace.showErrorMessage(Strings.CookieExpiredMessage);
		}

		// avoid function not defined
		window["onBack"] = window["swiped"] = window["onResume"] = window["onPause"] = function(){};
	}

	public demo(){
		LoginView._clickCount++;
		if (LoginView._clickCount >= 2) {
			$("#username").val("admin@partnerautomation15.onmicrosoft.com");
			$("#password").val("Password!1")
		}
	}

	public getInstallUrl(){
		if (isDebug()){
			if (ViewModelBase.device.winPhone)
				return "http://www.windowsphone.com/s?appid=c513d238-96c6-4687-97d0-1b927f913016"; // todo: release a hidden edog on wp store
			else if (ViewModelBase.device.android)
				return "https://pacmobile-edog.azurewebsites.net/debug/O365Partner.apk";
			else if (ViewModelBase.device.iOS)
				return "itms-services://?action=download-manifest&url=https://pacmobile-edog.azurewebsites.net/debug/O365Partner.plist";
			else
				return "";
		}
		else {
			if (ViewModelBase.device.winPhone)
				return "http://www.windowsphone.com/s?appid=b6bab53e-dfac-470e-965d-c87eeeceb7c9";
			else if (ViewModelBase.device.android)
				return "https://play.google.com/store/apps/details?id=com.ms.o365partner";
			else if (ViewModelBase.device.iOS)
				return "https://itunes.apple.com/us/app/partner-admin-for-office-365/id923241552?ls=1&mt=8";
			else
				return "";
		}
	}

	// return if handled
	public onAjaxError(statusCode, statusDescription, error, api, blockUI):boolean {
		switch (statusCode) {
			case 500:
				Trace.showErrorMessage( Strings.InternalServerErrorLogin);
				return false; // send this to server

			case 401:
				//Fault Code: 0x80041012 Fault Message: The entered and stored passwords do not match.
				//Fault Code: 0x80041084 Fault Message: The password has to be changed.
				//Fault Code: 0x80041082 Fault Message: The password for the account has expired.

				if (statusDescription.indexOf("0x80041084") > -1 || statusDescription.indexOf("0x80041082") > -1) {
					Trace.showErrorMessage(Strings.PasswordNeedChange);
					return true;
				}
				else if (statusDescription.indexOf("0x80041012") > -1
					|| statusDescription.indexOf("Bad username/password") > -1
					|| statusDescription.indexOf("Unauthorized")
					|| statusDescription.indexOf("credentials can't be autheticated or authorized"))
				{
					Trace.showErrorMessage(Strings.InvalidUsernamePasswordMessage); // this include both wrong app password and master password
					return true;
				}
				else if (statusDescription.indexOf("0x800434d4") > -1) {
					Trace.showErrorMessage(Strings.AppPasswordNeeded); // mfa enabled but app password not set
					return true;
				}
				else if (statusDescription.indexOf("0x80041034") > -1) {
					Trace.showErrorMessage(Strings.InvalidUserName); // email not in right format
					return true;
				}
				else if (statusDescription.indexOf("SAML Token missing in SOAP response")>-1){ // this is hotmail account
					Trace.showErrorMessage(Strings.InvalidUserName);
					return true;
				}
				else if (statusDescription.indexOf("Server Unavailable") > -1) { // org id busy
					Trace.showErrorMessage(Strings.CannotReachService);
					return true;
				}
				else if (statusDescription.indexOf("IE10")>-1){ // IE10 bug
					Trace.showErrorMessage(Strings.InvalidUsernamePasswordMessage);
					return true;
				}
				else {
					Trace.showErrorMessage(Strings.InvalidUsernamePasswordMessage);
					return false; // send this specific acces denied message to server
				}

			case 403:
				Trace.showErrorMessage( Strings.AdminNeededMessage);
				return true;

			default:
				return super.onAjaxError(statusCode, statusDescription, error, api, blockUI);
		}
	}

	public clickLogin() {
		Trace.clearErrorMessage();

		var userName = $("#username").val().trim();
		var password = $("#password").val();

		//Validate user inputs
		if (userName == "" || password == "") {
			Trace.showErrorMessage(Strings.InvalidUsernamePasswordMessage);
		}
		else if (userName.indexOf(" ") != -1) {
			Trace.showErrorMessage(Strings.InvalidUserNameWithSpaceMessage);
		}
		else if (userName.indexOf("@") == -1) {
			Trace.showErrorMessage(Strings.InvalidUserName);
		}
		else {
			var _this = this;
			this.asyncCheckTime();
			this.asyncRegister(userName, password);
		}
	}

	public asyncRegister(userName, password) {

		/* ----- Putting \\\\ as regular expression consider that as \\ double back slash ------------*/
		var replace = new Array("\\\\", '"');
		var replaceBy = new Array("\\\\", '\\"');
		for (var i = 0; i < replace.length; i++) {
			var regex = new RegExp(replace[i], "g");
			password = password.replace(regex, replaceBy[i]);
			userName = userName.replace(regex, replaceBy[i]);
		}

		// change username earlier to enable logging for failed login
		if (userName != this.preferences.userName) {
			Trace.logInfo("User changed, so clear cache.");
			this.cacheLocal.clearUserDataAndPreferences();
			this.tenantData.clear();
		}

		this.preferences.userName = userName;
		this.userData.cookieExpired = false;

		var serviceParams = {
			userName: userName,
			password: password,
			domains: ""
		};

		var _this = this;

		var callback = function (response : RegisterInfo) {
			if (_this.preferences.savePassword == true) {
				_this.preferences.password = password;
			}
			else{
				_this.preferences.password = "";
			}

			if ("UserRole" in response)
				_this.userData.userRole = response.UserRole;

			if ("PartnerRole" in response)
				_this.userData.partnerRole = response.PartnerRole;

			_this.preferences.userId = response.UserId;
			_this.userData.cookie = response.RegistrationCookie;
			_this.userData.lastCookieRefreshMilliSec = (new Date()).getTime();
			// Go to default pivot always after relogin
			_this.userData.lastPrimaryPivot = "";
			_this.userData.lastSecondPivot = "";
			_this.blockUIAjaxCount = Number.MAX_VALUE; // Continue to show progress bar until new page is loaded.
			_this.navigateTo("PivotView.html");
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.registerTag,
			serviceParams,
			callback,null,true);

	}
	
	public localeChanged():boolean { // noteused
		var userLocale = NativeHelper.getUserLocale();//window.navigator.userLanguage || window.navigator.language;
		return (this.userData.cookie && userLocale != this.userData.locale);
	}
}

