﻿/// <reference path="Utilities.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="ViewModelBase.ts" />
/// <reference path="LoginView.ts" />
/// <reference path="ClientsView.ts" />
/// <reference path="UsersView.ts" />
/// <reference path="HealthView.ts" />
/// <reference path="SupportView.ts" />
/// <reference path="MessagesView.ts" />
/// <reference path="SettingsView.ts" />
/// <reference path="jquerymobile.d.ts" />
/// <reference path="TenantCommunication.d.ts" />

class PivotView extends ViewModelBase {
	public static instance : PivotView;
	public panelClosed = true;
	public menuItems = [ClientsView, HealthView, UsersView, SupportView, MessagesView, FeedbackView, LoginView];
	public currentPivot:ViewModelBase = null;
	public get pincodeView():PincodeView {return <PincodeView> this.getInstance(PincodeView);}
	private _refreshTimeoutHandle = null;

	constructor() {
		super(true);

		this.initEventLogging() // make sure flurry is ready before sub views
		ViewModelBase.logCapptainStartActivity("SignIn");

		if (!this.userData.cookie)
			this.signOut();

		if (!this.isPartner()) { // hide partner pivot
			var index = this.menuItems.indexOf(ClientsView);
			this.menuItems.splice(index, 1);
		}

		if (this.userData.selectedIncidentId)
			this.userData.clearClientList();

		this.onHashChange(); // got to pivot if it is in url

		if (!this.currentPivot)
			this.gotoDefaultPivot();

		this.bind(); // This need to be called after fields created
		this.loadViews();

		var _this = this;
		$("#nav-panel")["panel"]({
			beforeclose: function (event, ui) {
				_this.panelClosed = true;
			}
		});

		$("#nav-panel")["panel"]({
			beforeopen: function (event, ui) {
				_this.panelClosed = false;
			}
		});

		/* Listen for orientation changes
		 window.addEventListener("orientationchange", function () {
		 $('meta[name=viewport]').attr("content", "width=device-width,target-densitydpi=high-dpi, initial-scale=1, maximum-scale=1, user-scalable=0");
		 $(".AppBar").css("width", (window.innerWidth) + "px");
		 }, false);*/

		if (!this.cacheLocal.nativeSwipe || !Config.allowNativeSwipe) {
			// this does not work in android
			$(document).on("swiperight", this.openPanel.bind(this));
			$(document).on('swipeleft', '.ui-panel-dismiss', this.closePanel.bind(this));
			$(document).on('taphold', '.ui-panel-dismiss', this.closePanel.bind(this));
		}
		else {
			window["swiped"] = this.onNativeSwipe.bind(this);
		}

		// overwrite base
		window["onResume"] = this.onResume.bind(this);
		window["onPause"] = this.onPause.bind(this);
		window["onBack"] = this.onBack.bind(this);

		window.addEventListener("hashchange", this.onHashChange.bind(this), false);
		window["setLoadedFromNotification"] = this.setLoadedFromNotification.bind(this); // for future use
		$(document).on('click', '.ui-input-clear', this.onSearchClearClick.bind(this));
		$(document).on('blur', '#search', this.onSearchBlur.bind(this));
		$(".ui-panel-dismiss").bind('touchmove', function (e) {
			e.preventDefault()
		});

		// this does not work since closePanel cause UI change and may accidentally click something under panel-dismiss
		//$(document).on('vclick', '.ui-panel-dismiss', this.closePanel.bind(this));
	}

	public loadViews(){
		var views = $(".pivotItem");

		for (var i = 0; i < views.length; i++) {
			if (views[i].children.length == 0) {
				var viewName = views[i].id;
				assert(typeof window[viewName] == "function");
				this.loadView(window[viewName]);
			}
		}
	}

	public initEventLogging() {

		var uid: string = "N/A";

		try {
			uid = CryptoJS.SHA256(this.preferences.userId).toString(CryptoJS.enc.Base64);
		}
		catch (e) {
			Trace.logError(e);
		}

		// optics
		if (FlurryAgent && FlurryAgent.startSession) {
			if (isDebug() || this.isDebugUser())
				FlurryAgent.startSession("BD229KWJ3672J99CSDY4"); // edog
			else
				FlurryAgent.startSession("3D9KBP8GS4VHNWPSBKS8");

			if (isDebug())
				FlurryAgent.setRequestInterval(5); // minimum 5 seconds

			FlurryAgent.setAppVersion(ViewModelBase.getAppVersion());
			FlurryAgent.setUserId(uid);
			Trace.logInfo("flurry inited");
		}

		if (capptain) {
			if (isDebug() || this.isDebugUser())
				capptain.appId = "opa000002"; // AppId for EDog 
			else
				capptain.appId = "opa000001"; // AppId for PROD

			var appInfos = {
				userid: uid,
				OS: ViewModelBase.getDeviceOS(),
				userAgent: navigator.userAgent
			};
			ViewModelBase.logCapptainAppInfo(appInfos);
			Trace.logInfo("capptain inited");
		}
	}

	public isDebugUser():boolean {
		var debugUsers = [
			"d12bc342-9a92-44e1-bd69-5948519c8487", //admin@partnerautomation15.onmicrosoft.com
			"c3374beb-bd67-48db-8bf9-fb2842f72f66", //admin@TestTestMattwillOldExp4.onmicrosoft.com
			"0777fab6-8163-40a8-aeeb-67cbbac709ca", //admin@TESTTESTdavitLargeP4.onmicrosoft.com
			"da5aa0a6-447a-4218-91c8-2d71dd4db7cf" //jblack@TESTTESTdavitLargeP4.onmicrosoft.com
		]

		return debugUsers.indexOf(this.preferences.userId) > -1;
	}

	public onSearchClearClick() {
		this.currentPivot.onSearchClearClick();
	}

	public onSearchBlur() {
		this.currentPivot.onSearchBlur();
	}

	public goBack():string {
		if (window.history.length >= 2) { // Do not go back to login page
			Trace.clearErrorMessage();
			window.history.back();
			return window.location.href;
		}
		else {
			return ""; // quite app
		}
	}

	public getTitle():string {
		if (this.currentPivot.getTitle())
			return this.currentPivot.getTitle();
		else
			return ViewModelBase.capitalize(this.currentPivot.getClassName().replace("View",""));
	}

	public getSecondaryTitle():string {
		if (this.currentPivot.getSecondaryTitle())
			return this.currentPivot.getSecondaryTitle();
		else {
			if (this.currentPivot instanceof ClientsView
				|| this.currentPivot instanceof SettingsView
				|| this.currentPivot instanceof SyncView
				|| this.currentPivot instanceof FeedbackView
				|| this.currentPivot instanceof  PincodeView)
				return "";
			else if (this.tenantData.isAOBO && this.tenantData.companyInfo)
				return this.tenantData.companyInfo.DisplayName;
			else
				return this.getUserDomain();
		}
	}

	public getUserDomain():string {
		return this.preferences.userName.toLowerCase().split("@")[1];
	}

	public getRightBtn():string {
		return this.currentPivot.getRightBtn();
	}

	public clickRightBtn() {
		this.currentPivot.clickRightBtn();
	}

	public getLeftBtn():string {
		return this.currentPivot.getLeftBtn();
	}

	public clickLeftBtn() {
		this.currentPivot.clickLeftBtn();
	}

	public onHashChange() {
		var hash = window.location.hash.split("&")[0].split("?")[0];
		hash = hash.replace("#", "");

		if (hash) {
			this.goToPivot(hash);
		}

		this.closePanel();
	}

	public setLoadedFromNotification() {

	}

	public onResume() {
		this.pincodeView.onBlur(); // for ios webview
	}

	public  onPause() {
		this.pincodeView.onBlur(); // for ios webview
	}

	public onBack():string {
		return  this.currentPivot.goBack();
	}

	// native swipe
	public onNativeSwipe(toRight) {
		this.setPanel(toRight);
	}

	public closePanel() {
		this.setPanel(false);
	}

	public openPanel() {
		this.setPanel(true);
	}

	public setPanel(open) {
		if (!$("#nav-panel")["panel"])
			return; // jqm not ready 

		if (open && this.currentPivot.menuEnabled) {
			$("#nav-panel")["panel"]("open");
		}
		else {
			$("#nav-panel")["panel"]("close");
		}
	}

	public clickMenuItem(viewClass: typeof ViewModelBase) {
		var id = this.getIconName(viewClass);
		var className = this.getClassName(viewClass);

		switch (viewClass) {
			case LoginView:
				this.logEvent("SignOut"); // log before signout
				this.signOut();
				break;
			case ClientsView:
				if (this.userData.selectedIncidentId) {
					this.userData.clearClientList();
					if (this.currentPivot instanceof ClientsView)
						this.getInstance(ClientsView).onReload(); // force reload
				}

			default :
				this.goTo(viewClass);
				this.logEvent(id);
		}

		$("#nav-panel")["panel"]("close");
	}


	public getMenuItemClass(viewClass: typeof ViewModelBase){
		return this.getIconClass(this.getIconName(viewClass));
	}

	public getIconName(viewClass: typeof ViewModelBase){
		if (viewClass == LoginView)
			return "Sign out"
		else
			return this.getClassName(viewClass).replace("View","")
	}

	public redirectToFeedbackPage() {
		this.goTo(FeedbackView);
	}

	public getMenuImageURL(pivot) {
		return "images/" + pivot.toLowerCase() + ".png";
	}

	public getMenuItemBackgroundColor() {
		return "#58595b";
	}

	public gotoLastPivot() { // not used

		if (this.userData.lastPrimaryPivot)
			this.goToPivot(this.userData.lastPrimaryPivot);
		else
			this.gotoDefaultPivot();
	}

	public gotoDefaultPivot() {
		if (this.isPartner())
			this.goTo(ClientsView);
		else
			this.goTo(HealthView);
	}

	public goToPivot(viewName:string) {
		Trace.logVerbose("try go to " + viewName);
		assert(viewName);
		if (typeof window[viewName] == "function"){
			this.goTo(window[viewName]);
		}
	}

	public goTo(viewClass: typeof ViewModelBase)
	{
		var viewName = this.getClassName(viewClass);
		if (this.currentPivot instanceof viewClass) {
			Trace.logVerbose("skipped " + viewName + " since no pivot change")
			return;
		}

		ViewModelBase.logCapptainEndActivity();
		ViewModelBase.logCapptainStartActivity(viewName);

		Trace.clearErrorMessage();

		if (viewClass == ClientsView) {
			if (this.tenantData.isAOBO)
				this.tenantData.clear();
		}

		var newPivot = this.getInstance(viewClass, true);

		if (newPivot.isPrimary()){
			this.userData.lastPrimaryPivot = viewName;
			this.userData.lastSecondPivot = "";
		}
		else{
			this.userData.lastSecondPivot = viewName;
		}

		if (this.currentPivot && this.currentPivot.skipHistory)
			window.location.replace(window.location.href.split('#')[0] + '#' +viewName);
		else
			window.location.hash = viewName;

		this.cacheLocal.save();

		$(".ui-pivot-items").show();
		$(".ui-pivot-items").children(".pivotItem.current").removeClass("current");
		var current = $(".ui-pivot-items").children("#" + viewName).addClass("current");
		if (this.currentPivot && newPivot.pageAnimation())
			current.css({ marginLeft: (-window.outerWidth), opacity: .5 })
				.animate({ marginLeft: "0px", opacity: 1 }, 200, 'linear');
		else
			current.animate({ marginLeft: "0px", opacity: 1 });

		// TODO: ios8 undefined is not an object (evaluating '$.mobile.silentScroll')
		$.mobile.silentScroll(0);

		this.currentPivot = newPivot;
		this.refreshIfNeeded();
		Trace.logVerbose("Went to " + viewName);
	}

	public asyncSearchServiceRequests() {

		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantDomains: []
		};

		if (this.tenantData.isAOBO)
			serviceParams.tenantDomains.push(this.tenantData.companyInfo.InitialDomain);

		var callback = function (response: ServiceRequest[]) {
			// items in array need to be tracked before binding, otherwise item property change will not update binding
			response.forEach(function (sr) { ko.track(sr); }); 

			this.tenantData.filteredSRs = response.sort(function (a, b) {
				return ViewModelBase.getDate(a.CreatedOn).getTime() - ViewModelBase.getDate(b.CreatedOn).getTime();
			})
		}

		this.tenantData.lastEventRefreshMilliSec = (new Date()).getTime(); // update time even request failed

		this.asyncRequest(
			Config.serviceURL,
			Config.searchServiceRequestsTag,
			serviceParams,
			callback);
	}

	public asyncGetServiceInfo() {
		var serviceParams = {
			lastCookie: this.userData.cookie,
			tenantDomains: this.isAOBO() ? [this.tenantData.companyInfo.InitialDomain] : [],
			locale: this.userData.locale
		}


		var _this = this;
		var callback = function (response) {

			if (response && response.length) {
				_this.tenantData.services = response;
				_this.asyncGetEventsInfo(EventType.Full);
				//this.getServiceIncidents();
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			this.isAOBO() ? Config.getServiceInformationForTenantDomainsTag : Config.getServiceInformationTag,
			serviceParams,
			callback);
	}

	public asyncGetEventsInfo(eventType) {

		var serviceParams = {
			lastCookie: this.userData.cookie,
			preferredEventTypes: [eventType], // 0 full, 1 is incremental 2 for message center
			tenantDomains: this.isAOBO() ? [this.tenantData.companyInfo.InitialDomain] : [],
			pastDays: 1,
			locale: this.userData.locale
		}

		var _this = this;
		var callback = function (response) {
			if (!_this.isAOBO())
				var events = response.Events;
			else if (response.EventsByTenantDomain && response.EventsByTenantDomain[0])
				events = response.EventsByTenantDomain[0].Value;

			if (events) {
				if (eventType == EventType.MC) {
					_this.tenantData.mcMessages = events.sort(function (m1, m2) {
						return ViewModelBase.getDate(m2.StartTime).getTime() - ViewModelBase.getDate(m1.StartTime).getTime();
					})
				}
				else {
					events.forEach(function (e) {
						e.AffectedTenantCount = -1
					}); // -1 means unknown, 0 means global
					_this.tenantData.incidents = events.map(HealthView.toServiceIncident);
				}
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			this.isAOBO() ? Config.getEventsForTenantDomainsTag : Config.getEventstag,
			serviceParams,
			callback);
	}

	public asyncGetEventsForPartner() {
		var client = this.tenantData.isAOBO;

		var serviceParams = {
			lastCookie: this.userData.cookie,
			pastDays: 7, // keep consistent with portal
			locale: this.userData.locale
		}

		var _this = this;
		var callback = function (response:Incident[]) {
			_this.userData.clientIncidents = response.map(HealthView.toServiceIncident);
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.getEventsForPartnerTag,
			serviceParams,
			callback);
	}

	public onSearchSubmit(e) {
		var searchString = $("#search", e).val().trim();
		$('#search').blur();
		Trace.clearErrorMessage();//todo remove
		this.currentPivot.search(searchString, true);
		return false;
	}

	public showProgress():boolean {
		if (this.currentPivot.canRefresh && this.ajaxCount)
			return true;
		else
			return !!this.currentPivot.ajaxCount;
	}

	//method to populate last refresh time in UI
	public GetLastRefreshTimeText() {

		var minutes = this.getCurrentRefreshMinutes();

		var lastRefreshPrefix = "Last synced "

		if (this.failedAjaxCount)
			lastRefreshPrefix += "failed "

		if (minutes > 30) {
			return lastRefreshPrefix + "more than 30 minutes ago";
		}
		else {
			return lastRefreshPrefix + minutes + " minutes ago";
		}
	}

	public refreshIfNeeded() {
		if (this.ajaxCount) {
			Trace.logInfo("skip refresh since it is in progress");
			return;
		}

		/* not working
		if (document.visibilityState != "visible"){
			Trace.logInfo("skip refresh since app is paused");
			return;
		}
		*/

		var minutes = this.getCurrentRefreshMinutes();

		if (this._refreshTimeoutHandle != null) {
			clearTimeout(this._refreshTimeoutHandle);
		}

		if (this.preferences.refreshMinutes != 0 && minutes > this.preferences.refreshMinutes) {
			Trace.clearErrorMessage();
			this.refresh();
		}

		this._refreshTimeoutHandle = window.setTimeout(this.refreshIfNeeded.bind(this), 60000);
	}

	public refresh() {
		//Trace.clearErrorMessage();

		this.asyncCheckTime();

		// SR
		this.asyncSearchServiceRequests();

		// Health
		this.asyncGetServiceInfo();

		if (this.isPartnerNotAOBO()) {
			this.asyncGetEventsForPartner();
		}

		//this.asyncGetEventsInfo(EventType.Full);
		if (!this.tenantData.mcMessages.length)
			this.asyncGetEventsInfo(EventType.MC);

		this.failedAjaxCount = 0;
	}

	private _getMiddle(start, end, factor) {
		return Math.floor(start + (end - start) * factor);
	}
}



