var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var SettingsView = (function (_super) {
    __extends(SettingsView, _super);
    function SettingsView() {
        _super.call(this);
        this.enablePin = "Enable pincode";
        this.disablePin = "Disable pincode";
        this.menuItems = [SyncView.menuName, FeedbackView.menuName, this.enablePin, this.disablePin];
        this.canRefresh = false;
        this.bind();
    }
    SettingsView.prototype.buttonEnabled = function (index) {
        var btn = this.menuItems[index];
        assert(btn);
        switch (btn) {
            case this.disablePin:
                return !!this.preferences.pincode;
            case this.enablePin:
                return !this.preferences.pincode;
            default:
                return true;
        }
    };
    SettingsView.prototype.getRightBtn = function () {
        return "hidden";
    };
    SettingsView.prototype.clickPage = function (index) {
        assert(index >= 0);
        var btn = this.menuItems[index];
        switch (btn) {
            case SyncView.menuName:
                this.goTo(SyncView);
                this.logEvent("SyncFreqency");
                break;
            case FeedbackView.menuName:
                this.goTo(FeedbackView);
                this.logEvent("Feedback");
                break;
            case this.disablePin:
                this.preferences.pincode = "";
                this.logEvent("DisablePin");
                break;
            case this.enablePin:
                this.goTo(PincodeView);
                this.logEvent("EnablePin");
                break;
            default:
        }
        $("#nav-panel")["panel"]("close");
    };
    SettingsView.menuName = "Settings";
    return SettingsView;
})(ViewModelBase);
var SyncView = (function (_super) {
    __extends(SyncView, _super);
    function SyncView() {
        _super.call(this);
        this.intervals = [15, 30, 0];
        this.canRefresh = false;
        this.bind();
    }
    SyncView.prototype.getTitle = function () {
        return "Sync Frequency";
    };
    SyncView.prototype.getRightBtn = function () {
        return "hidden";
    };
    SyncView.prototype.getRadioClass = function (value) {
        var active = (value == this.preferences.refreshMinutes);
        return this.getIconClass(active ? "circleFill" : "circleEmpty");
        ;
    };
    SyncView.prototype.getRadioText = function (value) {
        if (value == 0)
            return "Manually";
        else
            return "Every " + value + " minutes";
    };
    SyncView.prototype.clickRadio = function (value) {
        this.preferences.refreshMinutes = value;
        this.cacheLocal.save();
    };
    SyncView.menuName = "Sync Frequency";
    return SyncView;
})(ViewModelBase);
var PincodeView = (function (_super) {
    __extends(PincodeView, _super);
    function PincodeView() {
        _super.call(this);
        this.canRefresh = false;
        this.attempts = PincodeView.maxAttempts;
        this.menuEnabled = false;
        this.tempPin = "";
        this.bind();
        var onblur = this.onBlur.bind(this);
        var hidden = "hidden";
        if (hidden in document)
            document.addEventListener("visibilitychange", onblur);
        else if ((hidden = "mozHidden") in document)
            document.addEventListener("mozvisibilitychange", onblur);
        else if ((hidden = "webkitHidden") in document)
            document.addEventListener("webkitvisibilitychange", onblur);
        else if ((hidden = "msHidden") in document)
            document.addEventListener("msvisibilitychange", onblur);
        Trace.logInfo("onblur binded to " + hidden);
    }
    Object.defineProperty(PincodeView.prototype, "storedPin", {
        get: function () {
            return this.preferences.pincode;
        },
        set: function (val) {
            this.preferences.pincode = val;
        },
        enumerable: true,
        configurable: true
    });
    PincodeView.prototype.pageAnimation = function () {
        return !this.storedPin && _super.prototype.pageAnimation.call(this);
    };
    PincodeView.prototype.getTitle = function () {
        if (this.storedPin)
            return "Office365 Partner Admin";
        else
            return _super.prototype.getTitle.call(this);
    };
    PincodeView.prototype.onReload = function () {
        this.clearForm();
        this.attempts = PincodeView.maxAttempts;
    };
    PincodeView.prototype.getRightBtn = function () {
        return "hidden";
    };
    PincodeView.prototype.getLeftBtn = function () {
        if (this.storedPin)
            return "hidden";
        else
            return _super.prototype.getLeftBtn.call(this);
    };
    PincodeView.prototype.onBlur = function (e) {
        if (e === void 0) { e = null; }
        if (!this.userData.cookie)
            return;
        var twoHours = 2 * 60 * 60 * 1000;
        var twoDays = twoHours * 24;
        var sinceLogin = (new Date()).getTime() - this.userData.lastCookieRefreshMilliSec;
        if (this.storedPin && sinceLogin < twoDays) {
            ViewModelBase.getRoot().goTo(PincodeView);
        }
        else if (sinceLogin > twoHours) {
            this.signOut(true);
        }
    };
    PincodeView.prototype.addNumber = function (val) {
        if (this.tempPin.length == 4)
            return;
        this.tempPin += val;
        if (this.tempPin.length == 4)
            this.submitForm(this.tempPin);
        else
            $('#PINbox').val(this.tempPin);
    };
    PincodeView.prototype.clearForm = function () {
        this.tempPin = "";
    };
    PincodeView.prototype.submitForm = function (pincode) {
        if (this.storedPin) {
            if (this.storedPin == pincode) {
                ViewModelBase.getRoot().goBack();
                this.logEvent("CorrectPin");
            }
            else {
                this.logEvent("WrongPin");
                this.attempts--;
                if (this.attempts == 0)
                    this.signOut();
            }
            this.clearForm();
        }
        else {
            var _this = this;
            this.logEvent("SavePin");
            setTimeout(function () {
                _this.storedPin = pincode;
                ViewModelBase.getRoot().goBack();
                _this.clearForm();
            }, 1000);
        }
    };
    PincodeView.prototype.cancelForm = function () {
        this.clearForm();
        if (!this.storedPin) {
            ViewModelBase.getRoot().goBack();
            this.logEvent("CancelPin");
        }
        else {
            this.signOut();
        }
    };
    PincodeView.prototype.goBack = function () {
        if (!this.storedPin)
            return _super.prototype.goBack.call(this);
        else {
            Trace.logInfo("navigate back skipped at pincode view.");
            return window.location.href;
        }
    };
    PincodeView.prototype.getMessage = function () {
        if (this.storedPin && this.attempts < PincodeView.maxAttempts)
            return Strings.format(Strings.WrongPincode, this.attempts);
        else if (!this.storedPin && this.tempPin.length == 4)
            return Strings.PincodeSaved;
        else if (!this.storedPin)
            return Strings.ProvidePincode;
        else
            return Strings.PincodeUnlock;
    };
    PincodeView.maxAttempts = 5;
    return PincodeView;
})(ViewModelBase);
var FeedbackView = (function (_super) {
    __extends(FeedbackView, _super);
    function FeedbackView() {
        _super.call(this);
        this.canRefresh = false;
        this.pageIndex = 0;
        this.rating = 0;
        this.bind();
    }
    FeedbackView.prototype.onReload = function () {
        this.pageIndex = 0;
        this.rating = 0;
    };
    FeedbackView.prototype.like = function () {
        this.rating = 1;
    };
    FeedbackView.prototype.dislike = function () {
        this.rating = 2;
    };
    FeedbackView.prototype.submitFeedback = function () {
        var note = $("#newFeedback").val();
        this.logEvent("submitFeedback");
        if (note)
            this.asyncSendFeedback(this.rating, note);
        else
            Trace.showErrorMessage(Strings.EmptyMessage);
    };
    FeedbackView.prototype.asyncSendFeedback = function (rating, note) {
        var serviceParams = {
            userName: this.preferences.userName,
            apiName: "feedback" + rating,
            appVersion: ViewModelBase.getAppVersion(),
            operatingSystem: navigator.userAgent,
            exeptionDetails: note,
            callStacks: this.userData.cookie,
            deviceId: ""
        };
        var _this = this;
        var callback = function (xhr, result, error) {
            $("#newFeedback").val("");
            _this.pageIndex = 1;
        };
        this.asyncRequest(Config.serviceURL, Config.sendExceptiontag, serviceParams, callback, null, true);
    };
    FeedbackView.menuName = "Feedback";
    return FeedbackView;
})(ViewModelBase);
//# sourceMappingURL=SettingsView.js.map