﻿/// <reference path="Utilities.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="ViewModelBase.ts" />

class SettingsView extends ViewModelBase {
	public static menuName = "Settings";
	public enablePin = "Enable pincode";
	public disablePin = "Disable pincode";
	public menuItems = [SyncView.menuName, FeedbackView.menuName, this.enablePin , this.disablePin];
	public canRefresh = false;

	constructor() {
		super();

		this.bind();

	}

	public buttonEnabled(index:number) {
		var btn = this.menuItems[index];
		assert(btn);

		switch (btn) {
			case this.disablePin:
				return !!this.preferences.pincode;
			case this.enablePin:
				return !this.preferences.pincode;
			default :
				return true;
		}
	}

	public getRightBtn():string {
		return "hidden";
	}

	public clickPage(index:number) {
		assert(index >= 0);
		var btn = this.menuItems[index];

		switch (btn) {
			case SyncView.menuName:
				this.goTo(SyncView);
				this.logEvent("SyncFreqency");
				break;
			case FeedbackView.menuName:
				this.goTo(FeedbackView);
				this.logEvent("Feedback");
				break;
			case this.disablePin:
				this.preferences.pincode = "";
				this.logEvent("DisablePin");
				break;
			case this.enablePin:
				this.goTo(PincodeView);
				this.logEvent("EnablePin");
				break;
			default :
		}

		$("#nav-panel")["panel"]("close");
	}
}

class SyncView extends ViewModelBase {
	public static menuName = "Sync Frequency";
	public intervals = [15, 30, 0];
	public canRefresh = false;

	constructor() {
		super();
		this.bind();
	}

	public getTitle() {
		return "Sync Frequency";
	}

	public getRightBtn():string {
		return "hidden";
	}

	public getRadioClass(value) {
		var active = (value == this.preferences.refreshMinutes);

		return this.getIconClass(active ? "circleFill" : "circleEmpty");
		;
	}

	public getRadioText(value) {
		if (value == 0)
			return "Manually";
		else
			return "Every " + value + " minutes";
	}

	public clickRadio(value) {
		this.preferences.refreshMinutes = value;
		this.cacheLocal.save();
	}
}

class PincodeView extends ViewModelBase {
	public static maxAttempts = 5;
	public canRefresh = false;
	public attempts = PincodeView.maxAttempts;
	public menuEnabled = false;
	public tempPin = "";

	public get storedPin() {
		return this.preferences.pincode;
	}

	public set storedPin(val:string) {
		this.preferences.pincode = val;
	}

	constructor() {
		super();
		this.bind();

		var onblur = this.onBlur.bind(this);
		var hidden = "hidden";

		// Standards:
		if (hidden in document)
			document.addEventListener("visibilitychange", onblur);
		else if ((hidden = "mozHidden") in document)
			document.addEventListener("mozvisibilitychange", onblur);
		else if ((hidden = "webkitHidden") in document)
			document.addEventListener("webkitvisibilitychange", onblur);
		else if ((hidden = "msHidden") in document)
			document.addEventListener("msvisibilitychange", onblur);


		// All others:
		//window.onpagehide = onblur;  // this will trigger when press backspace to go back to login.html on android
		//window.onblur = onblur; // this will trigger when close dialog

		Trace.logInfo("onblur binded to " + hidden);
	}

	public pageAnimation() {
		return !this.storedPin && super.pageAnimation()
	}

	public getTitle() {
		if (this.storedPin)
			return "Office365 Partner Admin"
		else
			return super.getTitle();
	}

	public onReload() {
		this.clearForm();
		this.attempts = PincodeView.maxAttempts;
	}

	public getRightBtn():string {
		return "hidden";
	}

	public getLeftBtn():string {
		if (this.storedPin)
			return "hidden"
		else
			return super.getLeftBtn();
	}

	public onBlur(e = null) {
		if (!this.userData.cookie)
			return;

		var twoHours =  2 * 60 * 60 * 1000;
		var twoDays = twoHours * 24;
		var sinceLogin = (new Date()).getTime() - this.userData.lastCookieRefreshMilliSec;

		if (this.storedPin && sinceLogin < twoDays) { // if pin is enable, cookie expire time extends from two hours to two days
			ViewModelBase.getRoot().goTo(PincodeView);
		}
		else if (sinceLogin > twoHours){
			this.signOut(true);
		}
	}

	public addNumber(val) {
		if (this.tempPin.length == 4)
			return;

		this.tempPin += val;

		if (this.tempPin.length == 4)
			this.submitForm(this.tempPin);
		else
			$('#PINbox').val(this.tempPin);
	}

	public clearForm() {
		this.tempPin = ""
	}

	public submitForm(pincode) {
		if (this.storedPin) {
			if (this.storedPin == pincode) {
				ViewModelBase.getRoot().goBack();
				this.logEvent("CorrectPin");
			}
			else {
				this.logEvent("WrongPin");
				this.attempts--;
				if (this.attempts == 0)
					this.signOut();
			}
			this.clearForm();
		}
		else {
			var _this = this;
			this.logEvent("SavePin");
			setTimeout(function () {
				_this.storedPin = pincode;
				ViewModelBase.getRoot().goBack();
				_this.clearForm();
			}, 1000);
		}
	}

	public cancelForm() {
		this.clearForm();

		if (!this.storedPin) {
			ViewModelBase.getRoot().goBack();
			this.logEvent("CancelPin");
		}
		else {
			this.signOut();
		}
	}

	public goBack():string {
		if (!this.storedPin)
			return super.goBack();
		else {
			Trace.logInfo("navigate back skipped at pincode view.");
			return window.location.href;
		}
	}


	public getMessage() {
		if (this.storedPin && this.attempts < PincodeView.maxAttempts)
			return Strings.format(Strings.WrongPincode, this.attempts);
		else if (!this.storedPin && this.tempPin.length == 4)
			return Strings.PincodeSaved;
		else if (!this.storedPin)
			return Strings.ProvidePincode;
		else
			return Strings.PincodeUnlock
	}
}

class FeedbackView extends ViewModelBase {
	public static menuName = "Feedback";
	public canRefresh = false;
	public pageIndex = 0;
	public rating = 0;

	constructor() {
		super();
		this.bind();
	}

	public onReload() {
		this.pageIndex = 0;
		this.rating = 0;
	}

	public like() {
		this.rating = 1;
	}

	public dislike() {
		this.rating = 2;
	}

	public submitFeedback() {
		var note:string = $("#newFeedback").val();

		this.logEvent("submitFeedback");

		if (note)
			this.asyncSendFeedback(this.rating, note);
		else
			Trace.showErrorMessage(Strings.EmptyMessage)
	}

	public asyncSendFeedback(rating, note) {
		var serviceParams = {// todo: add status
			userName: this.preferences.userName,
			apiName: "feedback" + rating,
			appVersion: ViewModelBase.getAppVersion(),
			operatingSystem: navigator.userAgent,
			exeptionDetails: note,
			callStacks: this.userData.cookie,
			deviceId: ""
		}

		var _this = this;
		var callback = function (xhr, result, error) {
			$("#newFeedback").val("");
			_this.pageIndex = 1;
		}

		// Todo: need to handle error
		this.asyncRequest(
			Config.serviceURL,
			Config.sendExceptiontag,
			serviceParams,
			callback,
			null,
			true);
	}
}
