/// <reference path="Utilities.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="ViewModelBase.ts" />
/// <reference path="TenantCommunication.d.ts" />


class SupportView extends ViewModelBase {
	public static menuName = "Support";

	constructor() {
		super();

		this.bind(); // This need to be called after fields created
	}

	public isPrimary(){
		return true;
	}

	// Event Handler
	public clickServiceRequest(id) {
		this.tenantData.selectedSR = id;
		this.goTo(ServiceRequestView);
		this.logEvent("SR");
	}


	public getTicketClass(status) {
		if (status != "Open")
			status = "Closed";

		return this.getIconClass(status);
	}
}

class ServiceRequestView extends ViewModelBase {
	public canRefresh = false;
	public isEdit:boolean = false;
	public serviceRequest: ServiceRequest;
	constructor() {
		super();
		this.bind();
	}

	public onReload(){
		this.isEdit = false;
		this.serviceRequest = this.tenantData.selectedSR;
		assert(this.serviceRequest, "selectedSR is null");
		ko.track(this.serviceRequest);

		this.asyncGetServiceRequest(this.serviceRequest);
	}

	public getTitle():string{
		return this.tenantData.selectedSR.ServiceRequestNumber;
	}

	public getServiceRequest(){
		return this.tenantData.selectedSR;
	}

	public getRightBtn():string{
		if (!this.isEdit && this.editSRPermitted())
			return this.getIconClass("edit");
		else
			return "hidden";
	}

	public editSRPermitted():boolean{
		 return (!this.isAOBO() // currently non admin are not allowed to use this app, so if not aobo all admin can edit sr
			 || this.userData.partnerRole == PartnerRole.FullAdministration)  // limited and sale admin cannot edit client sr
	}

	public clickRightBtn(){
		this.isEdit = true;
		this.logEvent("EditSR");
	}

	public reopenSR(){
		this.logEvent("ReopenSR");
		this.asyncUpdateServiceRequest(this.serviceRequest, "Open", "")
	}

	public closeSR(){
		this.logEvent("CloseSR");
		this.asyncUpdateServiceRequest(this.serviceRequest, "Closed", "")
	}

	public addNote(){
		this.logEvent("AddSRNote");
		var note:string = $("#newNote").val();

		if (note)
			this.asyncUpdateServiceRequest(this.serviceRequest, "Unknown", note)
		else
			Trace.showErrorMessage(Strings.EmptyMessage)
	}

	// ajax calls
	public asyncGetServiceRequest(sr:ServiceRequest) {
		var serviceParams = {
			lastCookie: this.userData.cookie,
			referenceNumber: sr.ServiceRequestNumber
		};

		var _this = this;
		var callback = function (response : ServiceRequestDetails) {
			if (response) {
				if (response.Notes && response.Notes.length){
					response.Notes = response.Notes.sort(ServiceRequestView.compareNote);
				}
				sr.Details = response;
				_this.serviceRequest = _this.serviceRequest; // force ko update UI since detail is added after binding
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.getServiceRequestTag,
			serviceParams,
			callback);
	}

	public static compareNote(a, b){
		return ViewModelBase.getDate(b.CreatedDateTime).getTime() - ViewModelBase.getDate(a.CreatedDateTime).getTime();
	}

	public asyncUpdateServiceRequest(sr:ServiceRequest, status:string, note:string) {
		var serviceParams = {
			lastCookie: this.userData.cookie,
			referenceNumber: sr.ServiceRequestNumber,
			status: status,
			note: note
		};

		var _this = this;
		var callback = function (response) {
			$("#newNote").val("");

			if (response) {
				if (response.Notes && response.Notes.length) {
					response.Notes = response.Notes.sort(ServiceRequestView.compareNote);
				}
				sr.Details = response;
			}

			if (status != "Unknown") {
				sr.SRStatus = status;
			}
			else{ // addNote
				_this.isEdit = false;
			}
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.updateServiceRequestTag,
			serviceParams,
			callback, null, true);
	}
}
