

declare enum PartnerRole {
	/// <summary>
	/// No Partner administration role assigned
	/// </summary>
	None = 0,

	/// <summary>
	/// "Full administration" for Partner Admin Center role assigned
	/// </summary>
	FullAdministration = 1,

	/// <summary>
	/// "Limited administration" for Partner Admin Center role assigned
	/// </summary>
	LimitedAdministration = 2,

	/// <summary>
	/// "Sales support" for Partner Admin Center role assigned
	/// </summary>
	SalesSupport = 3
}

declare enum UserRole {
	/// <summary>
	///  No Role/ Not Used
	/// </summary>
	None = 0,

	/// <summary>
	/// Unauthenticated user
	/// </summary>
	Unauthenticated = 1,

	/// <summary>
	/// Tenant information worker
	/// </summary>
	InformationWorker = 2,

	/// <summary>
	/// Tenant administrator
	/// </summary>
	CustomerAdministrator = 3,

	/// <summary>
	/// Microsoft partner administrator
	/// </summary>
	MicrosoftPartner = 4,

	/// <summary>
	/// Microsoft internal user
	/// </summary>
	MicrosoftInternal = 5
}

declare enum EventType {
	Full = 0,
	Incremental = 1,
	MC = 2,
}

/// <summary> 
/// Service Status. 
/// </summary> 
/// <remarks> The enum values are ordered based on the order they should be shown in any client
/// The lower the value the higher the position in the list when the status of servcie or feature is shown</remarks>
declare enum Status
{
	/// <summary> 
	/// Service Interruption status 
	/// </summary> 
	ServiceInterruption = 0,

	/// <summary> 
	/// Service degradation status
	/// </summary> 
	ServiceDegradation = 1,

	/// <summary> 
	/// Restoring Service status 
	/// </summary> 
	RestoringService = 2,

	/// <summary>
	/// Extended Recovery status
	/// </summary>
	ExtendedRecovery = 3,

	/// <summary>
	/// Investigating status
	/// </summary>
	Investigating = 4,

	/// <summary>
	/// Service Restored status
	/// </summary>
	ServiceRestored = 5,

	/// <summary>
	/// Service Restored status
	/// </summary>
	FalsePositive = 6,

	/// <summary>
	/// PIR published status
	/// </summary>
	PIRPublished = 7,

	/// <summary> 
	/// Information unavailable status 
	/// </summary> 
	InformationUnavailable = 8,

	/// <summary> 
	/// Service operational status 
	/// </summary> 
	ServiceOperational = 99,

	/// <summary> 
	/// Client Only. Response is not received yet.
	/// </summary> 
	Unknown = 100,
}

declare enum DomainAuthenticationType {
	Managed = 0,
	Federated = 1
}

declare enum DomainStatus {
	Unverified = 0,
	Verified = 1,
	PendingDeletion = 2,
}

declare enum DomainCapabilities {
	None = 0,
	Email = 1,
	Sharepoint = 2,
	OfficeCommunicationsOnline = 4,
	SharepointDefault = 8,
	FullRedelegation = 16,
	SharePointPublic = 128,
	OrgIdAuthentication = 256,
	Yammer = 512,
	Intune = 1024,
	All = 1951,
}


declare enum SkuTargetClass {
	NotAvailable = 0,
	User = 1,
	Tenant = 2,
}

declare enum UserType {
	/// <summary>
	/// The flag for PublicFolder
	/// </summary>
	PublicFolder = 0x000000002,

	/// <summary>
	/// The flag for RemoteMailUser
	/// </summary>
	RemoteMailUser = 0x000000006,

	/// <summary>
	/// The flag for ConferenceRoomMailbox
	/// </summary>
	ConferenceRoomMailbox = 0x000000007,

	/// <summary>
	/// The flag for EquipmentMailbox
	/// </summary>
	EquipmentMailbox = 0x000000008,

	/// <summary>
	/// The flag for ACLabelMailboxUser
	/// </summary>
	ACLabelMailboxUser = 0x40000000,

	/// <summary>
	/// The flag for MailboxUser
	/// </summary>
	None = 0x00000000,

	/// <summary>
	/// The flag for ArbitrationMailbox
	/// </summary>
	ArbitrationMailbox = 0x0000000A,

	/// <summary>
	/// The flag for RoomList
	/// </summary>
	RoomList = 0x0000000F,

	/// <summary>
	/// The flag for TeamMailboxUser
	/// </summary>
	TeamMailboxUser = 0x00000010,

	/// <summary>
	/// The flag for ACLabelTeamMailboxUser
	/// </summary>
	ACLabelTeamMailboxUser = 0x40000010,

	/// <summary>
	/// The flag for ACLabelSyncedMailboxUser
	/// </summary>
	ACLabelSyncedMailboxUser = 0xC0000006,

	/// <summary>
	/// The flag for SyncedMailboxUser
	/// </summary>
	SyncedMailboxUser = 0x80000006,

	/// <summary>
	/// The flag for SyncedConferenceRoomMailbox
	/// </summary>
	SyncedConferenceRoomMailbox = 0x80000706,

	/// <summary>
	/// The flag for SyncedEquipmentMailbox
	/// </summary>
	SyncedEquipmentMailbox = 0x80000806,

	/// <summary>
	/// The flag for SyncedTeamMailboxUser
	/// </summary>
	SyncedTeamMailboxUser = 0x80001006,

	/// <summary>
	/// The flag for ACLabelSyncedTeamMailboxUser
	/// </summary>
	ACLabelSyncedTeamMailboxUser = 0xC0001006,

	/// <summary>
	/// The flag for SyncedRemoteMailUser
	/// </summary>
	SyncedRemoteMailUser = 0x80000606,

	/// <summary>
	/// The flag for ACLabelSyncedRemoteMailUser
	/// </summary>
	ACLabelSyncedRemoteMailUser = 0xC0000606,

	/// <summary>
	/// The flag for SyncedPublicFolder
	/// </summary>
	SyncedPublicFolder = 0x80000206,
}


interface ServiceInfo  {
	FeatureNames: string[];
	ServiceName: string;
}

interface AffectedServiceHealthStatus {
	InternalName:string;
	ServiceName: string;
	Status: Status;
	ServiceFeatureStatus: ServiceFeatureStatus[];
}
interface ServiceFeatureStatus
{
	FeatureName: string;
	InternalName: string;
	Status: Status;
}

interface Incident {
	__type: string;
	EndTime: string;
	Id: string;
	LastUpdatedTime: string;
	Messages: {
		MessageText:string;
		PublishedTime: string;
	}[];

	StartTime: string;
	Status: string;
	Title: string;
	AffectedServiceHealthStatus: AffectedServiceHealthStatus[];
	AffectedTenantCount:number;
}

interface MCMessage {
	__type: string;
	EndTime: string;
	Id: string;
	LastUpdatedTime: string;
	Messages: 
		{
			MessageText: string;
			PublishedTime: string;
		}[];
	StartTime: string;
	Status: string;
	Title: string;
	ActionRequiredByDate: any;
	ActionType: string;
	Category: string;
	ExternalLink: string;
	UrgencyLevel: string
}


interface Note
{
	CreatedByName:string;
	CreatedDateTime:string;
	Text:string;
}

interface ServiceRequest {
	AdditionalStatus: string;
	CreatedOn: string;
	ExternalTitle: string;
	LastModifiedOn: string;
	PrimaryContactFirstName: string;
	PrimaryContactLastName: string;
	SRStatus: string;
	ServiceRequestNumber: string;
	Details:ServiceRequestDetails;
}

interface ServiceRequestDetails {
	ExternalTitle: string;
	ServiceRequestNumber: string;
	AdditionalStatus: string;
	Attachments: any;
	CreatedOn: string;
	CustomerReportedProductId: string;
	Description: string;
	LastClosedDateTime: any;
	ModifiedOn: string;
	Notes: {CreatedByName;Text;CreatedDateTime:string}[];
	Organizations: {
		EngagementContexts: any;
		FaxNumber: any;
		Ids: {SourceSystem: number; Value: string}[];
		IsIdentified: boolean;
		Name: string;
		PartnerType: any;
		PrimaryAddress: any;
		PrimaryPhoneNumber: any;
		RelationType: number}[];
	Properties: {NameId: string; Value: string}[];
	Severity: string;
	Status: string;
	SupportTopic: {
		Id: string;
		Type: number;
		Value: any}
}

interface Company {
	CompanyName: string;
	PrimaryDomain: string;
	TenantId: string;

	// extra
	Affected:boolean;
}

interface CompanyInfo
{
	AllowAdHocSubscriptions: boolean;
	AllowEmailVerifiedUsers: boolean;
	AuthorizedServiceInstances: any[];
	AuthorizedServices: any[];
	City: string;
	CompanyDeletionStartTime;
	CompanyTags: any[];
	CompanyType: number;
	Country;
	CountryLetterCode: string;
	DapEnabled: boolean;
	DefaultUsageLocation;
	DirectorySynchronizationEnabled: boolean;
	DirectorySynchronizationStatus: number;
	DisplayName: string;
	InitialDomain: string;
	LastDirSyncTime;
	LastPasswordSyncTime;
	MarketingNotificationEmails: any[];
	ObjectId: string;
	PasswordSynchronizationEnabled: boolean;
	PortalSettings;
	PostalCode;
	PreferredLanguage: string;
	ReplicationScope: string;
	RmsViralSignUpEnabled: boolean;
	SelfServePasswordResetEnabled: boolean;
	ServiceInformation: any[];
	ServiceInstanceInformation: any[];
	ServiceInstanceEndpoints: any[]
	State: string;
	Street: string;
	TechnicalNotificationEmails: string[];
	TelephoneNumber;
	UIExtensibilityUris;
	UsersPermissionToCreateGroupsEnabled: boolean;
	UsersPermissionToCreateLOBAppsEnabled: boolean;
	UsersPermissionToReadOtherUsersEnabled: boolean;
	UsersPermissionToUserConsentToAppEnabled: boolean
}

interface Domain {
	Authentication: DomainAuthenticationType;
	Capabilities: DomainCapabilities;
	IsDefault: boolean;
	IsInitial: boolean;
	Name: string;
	RootDomain; 
	Status: DomainStatus;
	VerificationMethod: number
}


interface Sku 
{
	AccountName: string;
	AccountObjectId: string;
	AccountSkuId: string;
	ActiveUnits: number;
	SuspendedUnits: number;
	WarningUnits: number;
	ConsumedUnits: number;
	LockedOutUnits: number;
	ServiceStatus: ServiceStatus[];
	SkuId: string;
	SkuPartNumber: string;
	SubscriptionIds: string[];
	TargetClass: SkuTargetClass;
}

interface ServiceStatus {
	ProvisioningStatus: number;
	ServicePlan: {ServiceName: string; ServicePlanId: string; ServiceType: string; TargetClass: number}}


interface UserInfo {
	BlockCredential?;
	City?: string;
	Country?: string;
	Department?: string;
	DisplayName?: string;
	Errors?;
	Fax?: string;
	FirstName?: string;
	ImmutableId?;
	IsBlackberryUser?;
	IsLicensed?: boolean;
	LastDirSyncTime?;
	LastName?: string;
	LicenseReconciliationNeeded?: boolean;
	Licenses? : {
		AccountSku: {
			AccountName: string;
			SkuPartNumber: string};
		AccountSkuId: string;
		GroupsAssigningLicense: any[];
		ServiceStatus: ServiceStatus[]
	} [];
	LiveId?;
	MobilePhone?: string;
	ObjectId?: string;
	Office?;
	OverallProvisioningStatus?: number;
	PasswordNeverExpires?;
	PhoneNumber?: string;
	PostalCode?: string;
	PreferredLanguage?;
	SoftDeletionTimestamp?;
	State?: string;
	StreetAddress?: string;
	StrongPasswordRequired?;
	Title?;
	UsageLocation?: string;
	UserPrincipalName?: string;
	ValidationStatus?;
	AlternateEmailAddresses?: string[];
	CloudExchangeRecipientDisplayType?:UserType;
}

interface RegisterInfo {
	DomainToCompanyIdMapping: any[];
	Regions: string[];
	RegistrationCookie: string;
	ServiceEndPointMapping: any[];
	SupportedEventTypes: Number[];
	UserRole: UserRole;
	PartnerRole: PartnerRole;
	UserId: string;
}

