var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var User = (function () {
    function User() {
        this.DisplayName = null;
        this.FirstName = null;
        this.LastName = null;
        this.UserPrincipalName = null;
        this.UsageLocation = null;
    }
    return User;
})();
var UsersView = (function (_super) {
    __extends(UsersView, _super);
    function UsersView() {
        _super.call(this);
        this.canRefresh = false;
        this.searchPlaceHolder = "Search users";
        this.showToolbar = true;
        this.bind();
    }
    Object.defineProperty(UsersView.prototype, "searchString", {
        get: function () {
            return this.tenantData.userFilter;
        },
        set: function (filter) {
            this.tenantData.userFilter = filter;
        },
        enumerable: true,
        configurable: true
    });
    UsersView.prototype.onReload = function () {
        if (!this.tenantData.filteredUsers.length && !this.searchString)
            this.asyncSearchUsers("");
        this.setSearchString(this.searchString);
        if (!this.tenantData.domains.length)
            this.asyncListDomains();
        if (!this.tenantData.skus.length)
            this.asyncListAccountSkus();
        if (!this.tenantData.companyInfo)
            this.asyncGetCompanyInfo();
    };
    UsersView.prototype.isPrimary = function () {
        return true;
    };
    UsersView.prototype.getItems = function () {
        return this.rightToolbarSelected ? this.tenantData.recentUsers : this.tenantData.filteredUsers;
    };
    UsersView.prototype.search = function (searchString, blockUI) {
        this.asyncSearchUsers(searchString, blockUI);
        if (blockUI)
            this.logEvent("SearchUsers");
    };
    UsersView.prototype.getRightBtn = function () {
        if (this.isAddUserAllowed())
            return this.getIconClass("add");
        else
            return _super.prototype.getRightBtn.call(this);
    };
    UsersView.prototype.clickRightBtn = function () {
        if (this.isAddUserAllowed()) {
            var user = new User();
            user.DisplayName = "";
            user.UserPrincipalName = "";
            this.tenantData.selectedUser = user;
            this.goTo(EditUserView);
            this.logEvent("AddUser");
        }
        else {
            _super.prototype.clickRightBtn.call(this);
        }
    };
    UsersView.prototype.isAddUserAllowed = function () {
        var assignableDomains = this.tenantData.domains.filter(UsersView.isDomainAssignable);
        return assignableDomains.length && this.tenantData.skus;
    };
    UsersView.prototype.getSyncStatus = function (user) {
        if (UserDetailsView.isFederatedUser(user, this.tenantData.domains))
            return "Federated";
        else if (!this.tenantData.companyInfo)
            return "";
        else if (user.LastDirSyncTime && this.tenantData.companyInfo.DirectorySynchronizationEnabled)
            return Strings.UserStatusSynced;
        else
            return Strings.UserStatusInCloud;
    };
    UsersView.prototype.clickItem = function (user) {
        this.tenantData.selectedUser = user;
        this.addToRecent(user);
        this.goTo(UserDetailsView);
        this.logEvent("User");
    };
    UsersView.prototype.addToRecent = function (user) {
        for (var i = 0; i < this.tenantData.recentUsers.length; i++) {
            if (this.tenantData.recentUsers[i].ObjectId == user.ObjectId) {
                this.tenantData.recentUsers.splice(i, 1);
                break;
            }
        }
        this.tenantData.recentUsers.unshift(user);
    };
    UsersView.prototype.onAjaxError = function (statusCode, statusDescription, error, api, blockUI) {
        if (api == Config.searchUsersTag && statusCode == 401) {
            Trace.showErrorMessage(Strings.UserNotFound);
            return true;
        }
        else if (api == Config.getCompanyInfoTag && statusCode == 401) {
            this.tenantData.companyInfo = {};
            return true;
        }
        else {
            return _super.prototype.onAjaxError.call(this, statusCode, statusDescription, error, api, blockUI);
        }
    };
    UsersView.prototype.asyncListDomains = function () {
        var serviceParams = {
            lastCookie: this.userData.cookie,
            tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null
        };
        var _this = this;
        var onSuccess = function (response) {
            if (response) {
                this.tenantData.domains = response;
            }
        };
        this.asyncRequest(Config.serviceURL, Config.ListDomainsTag, serviceParams, onSuccess);
    };
    UsersView.prototype.asyncGetCompanyInfo = function () {
        var serviceParams = {
            lastCookie: this.userData.cookie,
            tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null
        };
        var _this = this;
        var onSuccess = function (response) {
            this.tenantData.companyInfo = response;
        };
        this.asyncRequest(Config.serviceURL, Config.getCompanyInfoTag, serviceParams, onSuccess);
    };
    UsersView.prototype.asyncListAccountSkus = function () {
        var serviceParams = {
            lastCookie: this.userData.cookie,
            tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null
        };
        var _this = this;
        var onSuccess = function (response) {
            if (response) {
                this.tenantData.skus = response;
            }
        };
        this.asyncRequest(Config.serviceURL, Config.listAccountSkusTag, serviceParams, onSuccess);
    };
    UsersView.prototype.asyncSearchUsers = function (filter, blockUI) {
        if (blockUI === void 0) { blockUI = false; }
        filter = filter.toLowerCase();
        Trace.logVerbose("clear filtered users");
        this.tenantData.filteredUsers.length = 0;
        var serviceParams = {
            lastCookie: this.userData.cookie,
            filter: filter,
            tenantDomains: this.tenantData.isAOBO ? [this.tenantData.companyInfo.InitialDomain] : []
        };
        var _this = this;
        var onSuccess = function (response) {
            if (response && response.length) {
                _this.tenantData.filteredUsers = response;
                _this.rightToolbarSelected = false;
                this.searchString = filter;
            }
            else {
                Trace.showErrorMessage("User not found");
            }
        };
        this.asyncRequest(Config.serviceURL, Config.searchUsersTag, serviceParams, onSuccess, null, blockUI);
    };
    UsersView.isDomainAssignable = function (domain) {
        return domain.Authentication != 1 /* Federated */ && domain.Status == 1 /* Verified */ && !(domain.Capabilities & 8 /* SharepointDefault */) && domain.Name.toLowerCase().indexOf(".mail.onmicrosoft.com") == -1;
    };
    UsersView.menuName = "Users";
    return UsersView;
})(ViewModelBase);
var UserDetailsView = (function (_super) {
    __extends(UserDetailsView, _super);
    function UserDetailsView() {
        _super.call(this);
        this.canRefresh = false;
        this.tempPassword = "";
        this.pageIndex = 0;
        this.bind();
    }
    UserDetailsView.prototype.onReload = function () {
        this.user = this.tenantData.selectedUser;
        assert(this.user, "user is null");
        this.tempPassword = "";
        this.pageIndex = 0;
        ko.track(this.user);
    };
    UserDetailsView.prototype.getRightBtn = function () {
        if (this.isEditAllowed())
            return this.getIconClass("edit");
        else
            return _super.prototype.getRightBtn.call(this);
    };
    UserDetailsView.prototype.clickRightBtn = function () {
        if (this.isEditAllowed()) {
            this.goTo(EditUserView);
            this.logEvent("EditUser");
        }
        else
            _super.prototype.clickRightBtn.call(this);
    };
    UserDetailsView.prototype.getUserType = function () {
        switch (this.user.CloudExchangeRecipientDisplayType) {
            case 2 /* PublicFolder */:
                return "Public folder";
            case 7 /* ConferenceRoomMailbox */:
                return "Conference room";
            case 8 /* EquipmentMailbox */:
                return "Equipment";
            case 0 /* None */:
                return "Shared mailbox";
            case 6 /* RemoteMailUser */:
            case null:
            case 1073741824 /* ACLabelMailboxUser */:
            case 3221225478 /* ACLabelSyncedMailboxUser */:
            default:
                return this.user.IsLicensed ? "Licensed" : "Unlicensed";
        }
    };
    UserDetailsView.prototype.isResetPasswordAllowed = function () {
        if (!this.user.ObjectId)
            return false;
        else
            return true;
    };
    UserDetailsView.prototype.isEditAllowed = function () {
        return Config.allowEditUser && !!this.tenantData.domains && !!this.tenantData.companyInfo && !!this.user.ObjectId;
    };
    UserDetailsView.isFederatedUser = function (user, domains) {
        if (!user.UserPrincipalName)
            return false;
        var userDomain = user.UserPrincipalName.split("@")[1];
        for (var i = 0; i < domains.length; i++) {
            if (domains[i].Name.toLowerCase() == userDomain.toLowerCase()) {
                var domain = domains[i];
                break;
            }
        }
        return domain && domain.Authentication == 1 /* Federated */;
    };
    UserDetailsView.prototype.getTitle = function () {
        return this.user.DisplayName;
    };
    UserDetailsView.prototype.clickResetPassword = function () {
        if (UserDetailsView.isFederatedUser(this.user, this.tenantData.domains)) {
            Trace.showErrorMessage(Strings.FederatedDomain);
        }
        else {
            this.pageIndex = 1;
            this.logEvent("ResetPassword");
        }
    };
    UserDetailsView.prototype.cancelResetPassword = function () {
        this.pageIndex = 0;
        this.logEvent("CancelResetPassword");
    };
    UserDetailsView.prototype.onAjaxError = function (statusCode, statusDescription, error, api, blockUI) {
        if (statusCode == 401) {
            Trace.showErrorMessage(Strings.AccessDenied);
            return true;
        }
        else if (statusCode == 400 && statusDescription.indexOf("UserConflictAuthenticationException") > -1) {
            Trace.showErrorMessage(Strings.FederatedDomain);
            return true;
        }
        else {
            return _super.prototype.onAjaxError.call(this, statusCode, statusDescription, error, api, blockUI);
        }
    };
    UserDetailsView.prototype.confirmResetPassword = function () {
        this.asyncResetPassword();
        this.pageIndex = 0;
        this.logEvent("ConfirmResetPassword");
    };
    UserDetailsView.prototype.getImageUrl = function (name) {
        name = name || "blank";
        return "images/" + name + ".png";
    };
    UserDetailsView.prototype.getName = function () {
        return (this.user.FirstName || "") + " " + (this.user.LastName || "");
    };
    UserDetailsView.prototype.getLocation = function () {
        if (this.user.City || this.user.State || this.user.PostalCode)
            return this.user.City + ' ' + this.user.State + " " + this.user.PostalCode;
        else
            return "";
    };
    UserDetailsView.prototype.asyncResetPassword = function () {
        var user = this.user;
        if (user.UserPrincipalName.toLowerCase() == this.preferences.userName.toLocaleLowerCase()) {
            Trace.showErrorMessage("You can't reset your own password");
            return;
        }
        var serviceParams = {
            lastCookie: this.userData.cookie,
            upn: user.UserPrincipalName,
            password: "",
            tenantDomains: this.tenantData.isAOBO ? [this.tenantData.companyInfo.InitialDomain] : []
        };
        var _this = this;
        var onSuccess = function (response) {
            _this.tempPassword = response;
        };
        this.asyncRequest(Config.serviceURL, Config.resetPasswordTag, serviceParams, onSuccess, null, true);
    };
    UserDetailsView.viewName = "userDetails";
    return UserDetailsView;
})(ViewModelBase);
var EditUserView = (function (_super) {
    __extends(EditUserView, _super);
    function EditUserView() {
        _super.call(this);
        this.canRefresh = false;
        this.skipHistory = true;
        this.bind();
    }
    EditUserView.prototype.onReload = function () {
        this.user = ViewModelBase.clone(this.tenantData.selectedUser);
        this.alias = this.user.UserPrincipalName.split("@")[0];
        assert(this.tenantData.companyInfo);
        assert(this.tenantData.domains);
        this.assignableDomains = this.tenantData.domains.filter(UsersView.isDomainAssignable).map(function (domain) {
            return domain.Name;
        });
        this.allSkus = ViewModelBase.clone(this.tenantData.skus);
        this.assignableSkus = this.allSkus.filter(function (sku) {
            ko.track(sku);
            return sku.TargetClass == 1 /* User */;
        });
        this.usageLocation = this.user.UsageLocation || (this.tenantData.companyInfo ? this.tenantData.companyInfo.CountryLetterCode : "") || "US";
        this.skus = [];
        this.oldSkus = [];
        if (this.user.Licenses) {
            for (var i = 0; i < this.user.Licenses.length; i++) {
                for (var j = 0; j < this.assignableSkus.length; j++) {
                    if (this.assignableSkus[j].AccountSkuId == this.user.Licenses[i].AccountSkuId) {
                        this.skus.push(this.assignableSkus[j]);
                        this.oldSkus.push(this.assignableSkus[j]);
                    }
                }
            }
        }
        else if (!this.user.ObjectId && this.assignableSkus.length == 1 && this.getAvailable(this.assignableSkus[0]) > 0) {
            this.clickSkuRadio(this.assignableSkus[0]);
        }
        this.domain = this.user.UserPrincipalName.split("@")[1] || this.tenantData.domains[0].Name;
    };
    EditUserView.prototype.getTitle = function () {
        if (this.user.ObjectId)
            return this.user.UserPrincipalName.split("@")[0];
        else
            return "Add User";
    };
    EditUserView.prototype.getSecondaryTitle = function () {
        if (this.user.ObjectId)
            return "@" + this.user.UserPrincipalName.split("@")[1];
        else
            return _super.prototype.getSecondaryTitle.call(this);
    };
    EditUserView.prototype.getRightBtn = function () {
        return this.getIconClass("save");
    };
    EditUserView.prototype.clickRightBtn = function () {
        this.submit();
        this.logEvent("SaveUser");
    };
    EditUserView.prototype.getLicenseLocation = function () {
        return "Licenses" + '(' + this.usageLocation + ')';
    };
    EditUserView.prototype.submit = function () {
        if (!this.user.DisplayName)
            Trace.showErrorMessage(Strings.EmptyDisplayName);
        else if (!this.alias)
            Trace.showErrorMessage(Strings.EmptyAlias);
        else if (!this.user.ObjectId && !this.lettersAndNumbers(this.alias))
            Trace.showErrorMessage(Strings.InvalidAlias);
        else {
            this.user.UsageLocation = this.usageLocation;
            this.user.UserPrincipalName = this.alias + "@" + this.domain;
            this.user.IsLicensed = !!this.skus.length;
            if (this.user.ObjectId)
                this.asyncSetUser(this.user);
            else
                this.asyncAddUser(this.user);
        }
    };
    EditUserView.prototype.lettersAndNumbers = function (s) {
        var exp = /^[0-9a-zA-Z]+$/;
        return s.match(exp);
    };
    EditUserView.prototype.getSkuRadioClass = function (value) {
        var active = (this.skus.indexOf(value) > -1);
        return this.getIconClass(active ? "circleFill" : "circleEmpty");
    };
    EditUserView.prototype.getSkuRadioText = function (value) {
        var available = this.getAvailable(value);
        if (available < 0)
            available = 0;
        var name = EditUserView._skuMap[value.SkuPartNumber];
        return name + " (" + available + " available)";
    };
    EditUserView.prototype.getAvailable = function (value) {
        var total = value.ActiveUnits + value.WarningUnits;
        return total - value.ConsumedUnits;
    };
    EditUserView.prototype.clickSkuRadio = function (value) {
        if (this.skus.indexOf(value) > -1) {
            this.skus.remove(value);
            value.ConsumedUnits--;
        }
        else if (this.getAvailable(value) <= 0) {
            Trace.showErrorMessage(Strings.NotEnoughLicense);
        }
        else {
            this.skus.push(value);
            value.ConsumedUnits++;
        }
    };
    EditUserView.prototype.asyncAddUser = function (user) {
        var licenses = this.skus.map(EditUserView.skuToLicense);
        var serviceParams = {
            lastCookie: this.userData.cookie,
            tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null,
            user: this.user,
            licenses: licenses
        };
        var _this = this;
        var onSuccess = function (response) {
            _this.tenantData.skus = _this.allSkus;
            _this.tenantData.selectedUser = _this.user;
            _this.goTo(UserDetailsView);
            var view = _this.getInstance(UserDetailsView);
            view.tempPassword = response;
        };
        this.asyncRequest(Config.serviceURL, Config.addUserTag, serviceParams, onSuccess, null, true);
    };
    EditUserView.prototype.licenseOnly = function () {
        return this.isDirsync() || UserDetailsView.isFederatedUser(this.user, this.tenantData.domains);
    };
    EditUserView.prototype.isDirsync = function () {
        return this.user.LastDirSyncTime && this.tenantData.companyInfo && this.tenantData.companyInfo.DirectorySynchronizationEnabled;
    };
    EditUserView.prototype.asyncSetUser = function (user) {
        var _this = this;
        var added = this.skus.filter(function (value) {
            return _this.oldSkus.indexOf(value) == -1;
        });
        var removed = this.oldSkus.filter(function (value) {
            return _this.skus.indexOf(value) == -1;
        });
        if (this.licenseOnly()) {
            var user = new User();
            user.ObjectId = this.user.ObjectId;
            user.UsageLocation = this.user.UsageLocation;
        }
        else {
            user = this.user;
        }
        var serviceParams = {
            lastCookie: this.userData.cookie,
            tenantId: this.isAOBO() ? this.tenantData.companyInfo.ObjectId : null,
            user: user,
            addLicenses: added.map(EditUserView.skuToLicense),
            removeLicenses: removed.map(EditUserView.skuToLicense)
        };
        var _this = this;
        var onSuccess = function (response) {
            _this.tenantData.skus = _this.allSkus;
            _this.user.Licenses = response.Licenses;
            var index = _this.tenantData.filteredUsers.indexOf(_this.tenantData.selectedUser);
            assert(index > -1);
            _this.tenantData.filteredUsers.replace(_this.tenantData.selectedUser, _this.user);
            _this.tenantData.recentUsers.replace(_this.tenantData.selectedUser, _this.user);
            _this.tenantData.selectedUser = _this.user;
            _this.goBack();
        };
        this.asyncRequest(Config.serviceURL, Config.SetUserTag, serviceParams, onSuccess, null, true);
    };
    EditUserView.skuToLicense = function (sku) {
        return { AccountName: sku.AccountName, SkuPartNumber: sku.SkuPartNumber };
    };
    EditUserView.prototype.onAjaxError = function (statusCode, statusDescription, error, api, blockUI) {
        if (statusCode == 400 && statusDescription.indexOf("UserAlreadyExistsException") > -1) {
            Trace.showErrorMessage(Strings.DuplicateUser);
            return true;
        }
        else if (statusCode == 400 && statusDescription.indexOf("InvalidLicenseConfigurationException") > -1) {
            Trace.showErrorMessage(Strings.ConflictLicense);
            return true;
        }
        else if (statusCode == 400 && blockUI) {
            Trace.showErrorMessage(Strings.InvalidInput);
        }
        else
            return _super.prototype.onAjaxError.call(this, statusCode, statusDescription, error, api, blockUI);
    };
    EditUserView.viewName = "EditUserView";
    EditUserView._skuMap = {
        AAD_BASIC: "Azure Active Directory Basic",
        AAD_PREMIUM: "Azure Active Directory Premium",
        RIGHTSMANAGEMENT: "Azure Active Directory Rights",
        RIGHTSMANAGEMENT_GOV: "Azure Active Directory Rights",
        RIGHTSMANAGEMENT_FACULTY: "Azure Active Directory Rights for Faculty",
        RIGHTSMANAGEMENT_STUDENT: "Azure Active Directory Rights for Students",
        CLOUD_BI_STANDALONE: "Business Intelligence Plan 1",
        CLOUD_BI_WITH_OFFICE: "Business Intelligence Plan 1 with Office",
        SKYPELYNCPSTNB: "Business Voice",
        ECAL_SERVICES: "ECAL Services (EOA, EOP, DLP)",
        ECAL_SERVICES_FACULTY: "ECAL Services (EOA, EOP, DLP) for Faculty",
        ECAL_SERVICES_GOV: "ECAL Services (EOA, EOP, DLP) for Government",
        ECAL_SERVICES_STUDENT: "ECAL Services (EOA, EOP, DLP) for Student",
        ENTERPRISECLOUDSUITE: "Enterprise Cloud Suite",
        EMS: "Enterprise Mobility Suite",
        "6a467665 - 16a3-4abd-9d7f-562741e04bbb": "ENTERPRISEPACK_HIGHLANDER_TEST",
        EXCHANGESTANDARD_ALUMNI: "Exchange (Plan 1) for Alumni",
        EXCHANGESTANDARD_FACULTY: "Exchange (Plan 1) for Faculty",
        EXCHANGESTANDARD_STUDENT: "Exchange (Plan 1) for Students",
        EXCHANGEENTERPRISE_FACULTY: "Exchange (Plan 2) for Faculty",
        EXCHANGEENTERPRISE_STUDENT: "Exchange (Plan 2) for Students",
        EXCHANGEARCHIVE: "Exchange Archiving",
        EXCHANGEARCHIVE_ADDON: "Exchange Archiving",
        EXCHANGEARCHIVE_ADDON_GOV: "Exchange Archiving (EOA) for Government for Exchange Customers",
        EXCHANGEARCHIVE_GOV: "Exchange Archiving (EOA) for Government for Exchange Server",
        EXCHANGEARCHIVE_FACULTY: "Exchange Archiving (Exchange Server) for Faculty",
        EXCHANGEARCHIVE_STUDENT: "Exchange Archiving (Exchange Server) for Students",
        EXCHANGEARCHIVE_ADDON_FACULTY: "Exchange Archiving (Exchange) for Faculty",
        EXCHANGEARCHIVE_ADDON_STUDENT: "Exchange Archiving (Exchange) for Students",
        EOP_ENTERPRISE_PREMIUM_FACULTY: "Exchange Enterprise CAL Services (EOP, DLP) for Faculty",
        EOP_ENTERPRISE_PREMIUM_GOV: "Exchange Enterprise Cal Services (EOP, DLP) for Government (Private)",
        EOP_ENTERPRISE_PREMIUM_STUDENT: "Exchange Enterprise CAL Services (EOP, DLP) for Students",
        EXCHANGESTANDARD_GOV: "Exchange for Government (Plan 1G)",
        EXCHANGEENTERPRISE_GOV: "Exchange for Government (Plan 2G)",
        EXCHANGEDESKLESS: "Exchange Kiosk",
        EXCHANGEDESKLESS_GOV: "Exchange Kiosk for Government",
        EXCHANGESTANDARD: "Exchange Plan 1",
        EXCHANGEENTERPRISE: "Exchange Plan 2",
        EXCHANGETELCO: "Exchange POP",
        EOP_ENTERPRISE_FACULTY: "Exchange Protection for Faculty",
        EOP_ENTERPRISE_GOV: "Exchange Protection for Government (Private)",
        EOP_ENTERPRISE_STUDENT: "Exchange Protection for Student",
        EXCHANGE_ONLINE_WITH_ONEDRIVE_LITE: "Exchange with OneDrive for Business",
        PSTNADDNUMBERS_PILOT: "Extra Telephone Numbers for Business Voice",
        FOPE_ENTERPRISE_PREMIUM_PILOT: "ForeFront Premium Pilot",
        FOPE_ENTERPRISE_PILOT: "ForeFront Standard Pilot",
        INTUNE_A: "Intune A Direct",
        INTUNE_STORAGE: "Intune Additional Storage",
        INTUNE_D: "Intune D",
        MCOIMP_FACULTY: "Lync (Plan 1) for Faculty",
        MCOIMP_STUDENT: "Lync (Plan 1) for Students",
        MCOSTANDARD_FACULTY: "Lync (Plan 2) for Faculty",
        MCOSTANDARD_STUDENT: "Lync (Plan 2) for Students",
        MCOVOICECONF: "Lync (Plan 3)",
        MCOIMP_GOV: "Lync for Government (Plan 1G)",
        MCOSTANDARD_GOV: "Lync for Government (Plan 2G)",
        MCOVOICECONF_GOV: "Lync for Government (Plan 3G)",
        MCOINTERNAL: "Lync Internal Incubation and Corp to Cloud",
        MCOIMP: "Lync Plan 1",
        MCOSTANDARD: "Lync Plan 2",
        MCOVOICECONF_FACULTY: "Lync Plan 3 for Faculty",
        MCOVOICECONF_STUDENT: "Lync Plan 3 for Students",
        MCOVOICEESS: "Lync-to-phone 'Add On' to P1",
        INTUNE_O365_STANDALONE: "Mobile Device Management for Office 365",
        TESTOFFICESKU: "MSODS",
        OFFICE_BASIC: "Office 365 Basic",
        O365_BUSINESS: "Office 365 Business",
        O365_BUSINESS_ESSENTIALS: "Office 365 Business Essentials",
        O365_BUSINESS_PREMIUM: "Office 365 Business Premium",
        DEVELOPERPACK: "Office 365 Developer",
        DEVELOPERPACK_GOV: "Office 365 Developer",
        DEVELOPERPACK_PILOT: "Office 365 Developer Pilot",
        DEVELOPERPACK_B_PILOT: "Office 365 Developer Technical Preview",
        EDUPACK_FACULTY: "Office 365 Education for Faculty",
        EDUPACK_STUDENT: "Office 365 Education for Students",
        ENTERPRISEPACK_OSLO_PILOT: "Office 365 Enterprise E3 with Oslo Pilot",
        EOP_ENTERPRISE: "Office 365 Exchange Protection Enterprise",
        EOP_ENTERPRISE_B_PILOT: "Office 365 Exchange Protection Enterprise Technical Preview",
        EOP_ENTERPRISE_PREMIUM: "Office 365 Exchange Protection Premium",
        EOP_ENTERPRISE_PREMIUM_B_PILOT: "Office 365 Exchange Protection Premium Technical Preview",
        STANDARDWOFFPACK_IW_FACULTY: "Office 365 Faculty Individual SignUp",
        DEVPACK_B_PILOT: "Office 365 for Developer",
        STANDARDPACK_GOV: "Office 365 for Government (Plan G1)",
        STANDARDWOFFPACK_GOV: "Office 365 for Government (Plan G2)",
        ENTERPRISEPACK_GOV: "Office 365 for Government (Plan G3)",
        ENTERPRISEWITHSCAL_GOV: "Office 365 for Government (Plan G4)",
        DESKLESSPACK_GOV: "Office 365 for Government (Plan K1G)",
        DESKLESSWOFFPACK_GOV: "Office 365 for Government (Plan K2G)",
        MIDSIZEPACK: "Office 365 MidSize Business",
        MIDSIZEPACK_LEGACY: "Office 365 MidSize Business",
        MIDSIZEPACKWITHEXCHANGEARCHIVE_ADDON: "Office 365 MidSize Business with Exchange Archiving",
        STANDARDPACK_FACULTY: "Office 365 Plan A1 for Faculty",
        STANDARDPACK_STUDENT: "Office 365 Plan A1 for Students",
        STANDARDWOFFPACK_FACULTY: "Office 365 Plan A2 for Faculty",
        STANDARDWOFFPACK_STUDENT: "Office 365 Plan A2 for Students",
        ENTERPRISEPACK_EDULRG: "Office 365 Plan A3 for Faculty",
        ENTERPRISEPACK_FACULTY: "Office 365 Plan A3 for Faculty",
        ENTERPRISEPACK_STUDENT: "Office 365 Plan A3 for Students",
        ACADEMYPACK_PILOT: "Office 365 Plan A3 Pilot",
        ACADEMYPACK_B_PILOT: "Office 365 Plan A3 Technical Preview",
        ENTERPRISEWITHSCAL_FACULTY: "Office 365 Plan A4 for Faculty",
        ENTERPRISEWITHSCAL_STUDENT: "Office 365 Plan A4 for Students",
        STANDARDPACK: "Office 365 Plan E1",
        STANDARDPACKWITHEXCHANGEARCHIVE_ADDON: "Office 365 Plan E1 with Exchange Archiving",
        STANDARDWOFFPACK: "Office 365 Plan E2",
        ENTERPRISEPACK: "Office 365 Plan E3",
        ENTERPRISEPACKLRG: "Office 365 Plan E3",
        ENTERPRISEPACK_PILOT: "Office 365 Plan E3 Pilot",
        ENTERPRISEPACK_B_PILOT: "Office 365 Plan E3 Technical Preview",
        ENTERPRISEPACKWITHOUTPROPLUS: "Office 365 Plan E3 w/o ProPlus",
        ENTERPRISEWITHSCAL: "Office 365 Plan E4",
        ENTERPRISEWITHSCALWITHOUTPROPLUS: "Office 365 Plan E4 w/o ProPlus",
        ENTERPRISEWITHSCAL_VOICECONF: "Office 365 Plan E5",
        ENTERPRISEPACKWITHOUTPROPLUS_GOV: "Office 365 Plan G3 w/o ProPlus",
        ENTERPRISEWITHSCALWITHOUTPROPLUS_GOV: "Office 365 Plan G4 w/o ProPlus",
        DESKLESSPACK: "Office 365 Plan K1",
        DESKLESSPACK_YAMMER: "Office 365 Plan K1 with Yammer",
        DESKLESSWOFFPACK: "Office 365 Plan K2",
        LITEPACK: "Office 365 Plan P1",
        LITEPACK_VOICEESS: "Office 365 Plan P1 Voice",
        PROFESSIONAL_B_PILOT: "Office 365 Plan P2 Technical Preview",
        PROFESSIONAL_PILOT: "Office 365 Professional P1 Pilot",
        OFFICESUBSCRIPTION_B_PILOT: "Office 365 ProPlus Preview",
        SHAREPOINT_APP_HOSTING_ADDON: "Office 365 SharePoint – App Hosting",
        SHAREPOINT_APP_HOSTING_ADDON_FACULTY: "Office 365 SharePoint – App Hosting",
        SHAREPOINT_APP_HOSTING_ADDON_GOV: "Office 365 SharePoint – App Hosting",
        SHAREPOINT_CLOUD_BI_ADDON: "Office 365 SharePoint – Cloud BI",
        SHAREPOINT_CLOUD_BI_ADDON_GOV: "Office 365 SharePoint – Cloud BI",
        DUET_ADDON: "Office 365 SharePoint – DUET",
        DUET_ADDON_GOV: "Office 365 SharePoint – DUET",
        DUETPACK_B_PILOT: "Office 365 SharePoint – DUET 2.0 Technical Preview",
        DUET_ADDON_FACULTY: "Office 365 SharePoint – DUET for Faculty",
        DUET_ADDON_STUDENT: "Office 365 SharePoint – DUET for Students",
        LITEPACK_P2: "Office 365 Small Business Premium",
        LITEPACK_P2_B_PILOT: "Office 365 Small Business Premium Preview",
        DUETPACK_PILOT: "Office 365 SPO-DUET Pilot",
        FIS_SUITE: "Office 365 SPO-FIS Pilot",
        PROJECTPACK_PILOT: "Office 365 SPO-Project Pilot",
        OFFICE_STARTER: "Office 365 Starter",
        STANDARDWOFFPACK_IW_STUDENT: "Office 365 Student Individual SignUp",
        OFFICESUBSCRIPTION: "Office Professional Plus",
        OFFICESUBSCRIPTION_FACULTY: "Office Professional Plus for Faculty",
        OFFICESUBSCRIPTION_GOV: "Office Professional Plus for Government (1 Year)",
        OFFICESUBSCRIPTION_STUDENT: "Office Professional Plus for Students",
        WACSHAREPOINTSTD_FACULTY: "Office Web Apps (Plan 1) For Faculty",
        WACSHAREPOINTSTD_STUDENT: "Office Web Apps (Plan 1) For Students",
        WACSHAREPOINTENT_FACULTY: "Office Web Apps (Plan 2) For Faculty",
        WACSHAREPOINTENT_STUDENT: "Office Web Apps (Plan 2) For Students",
        WACSHAREPOINTSTD_GOV: "Office Web Apps for Government with SharePoint Plan 1G",
        WACSHAREPOINTENT_GOV: "Office Web Apps for Government with SharePoint Plan 2G",
        WACSHAREPOINTSTD: "Office Web Apps with SharePoint Plan 1",
        WACSHAREPOINTENT: "Office Web Apps with SharePoint Plan 2",
        ONEDRIVESTANDARD: "OneDrive for Business (Plan 1)",
        WACONEDRIVESTANDARD: "OneDrive for Business (Plan 1) with Office Web Apps",
        WACONEDRIVESTANDARD_GOV: "OneDrive for Business (Plan 1G) with Office Web Apps for Government",
        ONEDRIVESTANDARD_GOV: "OneDrive for Business for Government (Plan 1G)",
        POWER_BI_STANDARD: "Power BI",
        POWER_BI_STANDALONE: "Power BI for Office 365",
        POWER_BI_ADDON: "Power BI for Office 365 Add-On",
        POWER_BI_ADDON_FACULTY: "Power BI for Office 365 Add-On for Faculty",
        POWER_BI_ADDON_STUDENT: "Power BI for Office 365 Add-On for Students",
        POWER_BI_STANDALONE_FACULTY: "Power BI for Office 365 for Faculty",
        POWER_BI_STANDALONE_STUDENT: "Power BI for Office 365 for Students",
        POWER_BI_INDIVIDUAL_USER: "Power BI for Office 365 Individual User Trial",
        POWER_BI_PRO: "Power BI Pro",
        PROJECTESSENTIALS: "Project Essentials",
        PROJECTESSENTIALS_GOV: "Project Essentials for Government",
        PROJECTONLINE_PLAN_1: "Project Plan 1",
        PROJECTONLINE_PLAN_1_ADDON: "Project Plan 1",
        PROJECTONLINE_PLAN_1_ADDON_GOV: "Project Plan 1",
        PROJECTONLINE_PLAN_1_GOV: "Project Plan 1",
        PROJECTONLINE_PLAN_1_ADDON_STUDENT: "Project Plan 1  for Students",
        PROJECTONLINE_PLAN_1_STUDENT: "Project Plan 1  for Students",
        PROJECTONLINE_PLAN_1_ADDON_FACULTY: "Project Plan 1 for Faculty",
        PROJECTONLINE_PLAN_1_FACULTY: "Project Plan 1 for Faculty",
        PROJECTONLINE_PLAN_2: "Project Plan 2",
        PROJECTONLINE_PLAN_2_GOV: "Project Plan 2",
        PROJECTONLINE_PLAN_2_STUDENT: "Project Plan 2  for Students",
        PROJECTONLINE_PLAN_2_FACULTY: "Project Plan 2 for Faculty",
        PROJECTONLINE_B_PILOT: "Project PPM Plan 1 Technical Preview",
        PROJECTPACK_B_PILOT: "Project PPM Plan 2 Technical Preview",
        PROJECTCLIENT: "Project Pro for Office 365",
        PROJECTCLIENT_GOV: "Project Pro for Office 365",
        PROJECTCLIENT_FACULTY: "Project Pro for Office 365 for Faculty",
        PROJECTCLIENT_STUDENT: "Project Pro for Office 365 for Students",
        PROJECTCLIENT_B_PILOT: "Project Professional Technical Preview",
        CONFPSTNADDON_PILOT: "PSTN Audio Conferencing for Lync Plan 2",
        SHAREPOINTSTANDARD_FACULTY: "SharePoint (Plan 1) for Faculty",
        SHAREPOINTSTANDARD_STUDENT: "SharePoint (Plan 1) for Students",
        SHAREPOINTSTANDARD_YAMMER: "SharePoint (Plan 1) with Yammer",
        SHAREPOINTENTERPRISE_FACULTY: "SharePoint (Plan 2) for Faculty",
        SHAREPOINTENTERPRISE_STUDENT: "SharePoint (Plan 2) for Students",
        SHAREPOINTENTERPRISE_YAMMER: "SharePoint (Plan 2) with Yammer",
        SHAREPOINTSTANDARD_GOV: "SharePoint for Government (Plan 1G)",
        SHAREPOINTENTERPRISE_GOV: "SharePoint for Government (Plan 2G)",
        SHAREPOINTDESKLESS: "SharePoint Kiosk",
        SHAREPOINTEXTRANET: "SharePoint Partner Access",
        SHAREPOINTEXTRANET_GOV: "SharePoint Partner Access",
        SHAREPOINTEXTRANET_ALUMNI: "SharePoint Partner Access for Alumni",
        SHAREPOINTEXTRANET_FACULTY: "SharePoint Partner Access for EDU",
        SHAREPOINTEXTRANET_PARENTS: "SharePoint Partner Access for Parents",
        SHAREPOINTSTANDARD: "SharePoint Plan 1",
        SHAREPOINTENTERPRISE: "SharePoint Plan 2",
        SHAREPOINTSTORAGE: "SharePoint Storage",
        SHAREPOINTSTORAGE_FACULTY: "SharePoint Storage",
        SHAREPOINTSTORAGE_GOV: "SharePoint Storage",
        SHAREPOINTSTORAGELITE: "SharePoint Storage for OfficeLite",
        SHAREPOINTWACADDON: "SharePoint WAC Edit",
        SMB_BUSINESS: "SMB Business",
        SMB_BUSINESS_ESSENTIALS: "SMB Business Essentials",
        SMB_BUSINESS_PREMIUM: "SMB Business Premium",
        PSTNTOLLNUMBERS_PILOT: "Toll Free Telephone Numbers for Businss Voice",
        VISIOCLIENT: "Visio Pro for Office 365",
        VISIOCLIENT_GOV: "Visio Pro for Office 365",
        VISIOCLIENT_FACULTY: "Visio Pro for Office 365 for Faculty",
        VISIOCLIENT_STUDENT: "Visio Pro for Office 365 for Students",
        VISIOCLIENT_B_PILOT: "Visio Professional Technical Preview",
        WIFIENTERPRISE: "WiFi",
        YAMMER_ENTERPRISE_STANDALONE: "Yammer Enterprise Standalone"
    };
    return EditUserView;
})(ViewModelBase);
//# sourceMappingURL=UsersView.js.map