/// <reference path="jquery.d.ts" />
/// <reference path="TenantCommunication.d.ts" />
/// <reference path="ViewModelBase.ts" />


class Config {
	public static uiVersion = "0.0.58";
	public static minVersion = 45;
	public static allowEditUser = true;
	public static allowPIISavedForDebug = true;
	public static allowNativeSwipe = false;
	public static allowAggregatedHealth = true;
	public static localizeDate = true;
	public static localizeTime = true;
	public static resultSize = 30; // todo: tc is not accepting this parameter yet
	public static useAjaxNavigation = false;
	public static remoteHost = "";
	public static pageAnimation = true;

	//public static serviceURL = "https://127.0.0.1/shdtenantcommunications.svc/";
	public static serviceURL = "https://office365servicehealthcommunications.cloudapp.net/shdtenantcommunications.svc/";
	//public static serviceURL = "https://8907fd23cb0d460bae48e941ba976df7.cloudapp.net/shdtenantcommunications.svc/";
	public static timeURL = "http://tenantcommunicationdemo.azurewebsites.net/";
	public static timeTag = "time.php";
	// api
	public static registerTag = "Register";
	public static findAffectedTenantsTag = "FindAffectedTenantGuids";
	public static searchCompanyInfoTag = "SearchCompanyInfo";
	public static getServiceInformationTag = "GetServiceInformation";
	public static getEventstag = "GetEvents";
	public static searchServiceRequestsTag = "SearchServiceRequests";
	public static searchUsersTag = "SearchUsers";
	public static resetPasswordTag = "ResetPassword";
	public static addUserTag = "AddUser";
	public static SetUserTag = "SetUser";
	public static getCompanyInfoTag = "GetCompanyInfo";
	public static listAccountSkusTag = "ListAccountSkus";
	public static ListDomainsTag = "ListDomains";
	public static getServiceRequestTag = "GetServiceRequest";
	public static updateServiceRequestTag = "UpdateServiceRequest";
	public static getPartnerFlaggedList = "GetPartnerFlaggedList";
	public static getEventsForTenantDomainsTag = "GetEventsForTenantDomains";
	public static getEventsForPartnerTag = "GetEventsForPartner";
	public static findAffectedTenantsForPartnerTag = "FindAffectedTenantsForPartner";
	public static getServiceInformationForTenantDomainsTag = "GetServiceInformationForTenantDomains";
	public static sendExceptiontag = "SendException";
}

class DeviceInfo {
	constructor() {
		Trace.logInfo("Useragent=" + navigator.userAgent);
	}

	public pixelRatio = window.devicePixelRatio || 1;
	public viewport = {
		width: window.innerWidth,
		height: window.innerHeight
	};
	public screen = {
		width: window.screen.availWidth * this.pixelRatio,
		height: window.screen.availHeight * this.pixelRatio
	};

	public userAgent = navigator.userAgent;
	public iPhone = /iPhone/i.test(navigator.userAgent);
	public iPad = /iPad/i.test(navigator.userAgent);
	public iOS = this.iPhone || this.iPad;

	public winPhone = /IEMobile/i.test(navigator.userAgent);
	public winTablet = this.IE && /Touch/i.test(navigator.userAgent);
	public IE = /Trident/i.test(navigator.userAgent);

	public android = /android/i.test(navigator.userAgent) && !this.winPhone; // wp8.1 also have android in useragent
	public androidTablet = this.android && !/Mobile/i.test(navigator.userAgent)

	public mobile = this.iOS || this.android || this.winPhone || /mobile/i.test(navigator.userAgent);
	public touchDevice = (('ontouchstart' in window) || (window.navigator.msMaxTouchPoints));
}

function assert(condition, msg = "assert error") {
	if (!condition) {
		var stacks = getStackTrace();
		var api = stacks.length > 1 ? stacks[1] : "Unknown";
		Trace.logExceptions(0, api, msg, JSON.stringify(stacks));
	}
};

module CryptoJS {
	export var SHA256: (message: string, key?: string, ...options: any[]) => any;
	export var enc: {
		Latin1;
		Utf8;
		Hex;
		Utf16;
		Base64;
	};
}

module FlurryAgent {

	export var startSession:Function;
	export var endSession:Function;
	export var pauseSession:Function;
	export var setRequestInterval:Function; //(timeInSeconds);
	export var setAppVersion:Function;
	export var setUserId:Function;
	export var logEvent:Function; // (eventName, eventParameters, true
	export var logError:Function; // (errorName, errorMessage, lineNumber);
}

function isDebug() : string {
    if (window.location.protocol.toLowerCase().indexOf("http") < 0 || window.location.href.toLowerCase().indexOf("localhost") > -1)
		return "D" // local debug
	else if (window.location.href.toLowerCase().indexOf("nauld") > -1 || window.location.href.toLowerCase().indexOf("edog") > -1)
		return "E" // edog
	else
		return "" // prd
}

class Strings {
	public static format(s, arg1, arg2 = null, arg3 = null) {
		var args = arguments;
		return s.replace(/\{\{|\}\}|\{(\d+)\}/g, function (curlyBrack, index) {
			return ((curlyBrack == "{{") ? "{" : ((curlyBrack == "}}") ? "}" : args[parseInt(index) + 1]));
		});
	}

	// Part 1: error messages

	public static ProvidePincode =
		"Please provide 4 digit pincode.";

	public static PincodeSaved =
		"Pincode has been set successfully.";

	public static PincodeUnlock =
		"Please enter your 4 digit pincode to begin.";

	public static WrongPincode =
		"Wrong pincode, {0} attempts remaining before being signed out.";

	public static NoServieRequestMessage =
		"No service request found.";

	public static CookieExpiredMessage =
		"Your session expired. You need to sign in again.";

	public static SubscriptionChangeMessage =
		"Your session expired. You need to sign in and update your app settings.";

	public static ServiceInfoChangeMessage =
		"Sorry; we made a change to the mobile app service. You might need to reconfigure your settings.";

	public static InternalAppErrorMessage =
		"Sorry; the app closed. You need to sign in again.";

	public static InternalServerErrorLogin =
		"Couldn’t sign in. Try again later.";

	public static InternalServerError =
		"Unknown error.";

	public static InvalidUserNameWithSpaceMessage =
		"That user name didn't work. Try again without using spaces.";

	public static InvalidUserName =
		"Couldn’t sign in. Please use Office 365 administrator ID assigned to you by your work or school.";

	public static InvalidUsernamePasswordMessage =
		"Couldn’t sign in. Check your email address and password.";

	public static AppPasswordNeeded =
		"Multi-factor authentication is enabled on your account. You need to set up app password at 'portal.office.com' before using this app.";

	public static PasswordNeedChange =
		"You must update your password at 'portal.office.com' before using this app because this is the first time that you've signed in or your password has expired.";

	public static AdminNeededMessage =
		"Only Office 365 administrators can use this app."

	public static ClientsNotFound =
		"Couldn’t find that client."

	public static ClientNotFound =
		"Client does not exist or you do not have permission."

	public static UserNotFound =
		"Couldn’t find that user."

	public static AccessDenied =
		"Access denied."

	public static DuplicateUser =
		"The email address is being used by another user."

	public static ConflictLicense =
		"Conflicting licenses are selected."

	public static InvalidInput =
		"Invalid input."

	public static NotEnoughLicense =
		"License is not available."

	public static SRNotFound =
		"Couldn’t find that service request."

	public static DomainInvalid = // not used
		"Domain name not in valid format."

	public static FederatedDomain =
		"Federated domain is not supported."

	public static InvalidSearchString =
		"Search string contains invalid characters."

	public static LocalChangedMessage =
		"You changed your phone's locale setting. You need to sign in again.";

	public static EmptyMessage =
		"Please write a few words before submit.";

	public static EmptyDisplayName =
		"Display name cannot be empty.";

	public static EmptyAlias =
		"User name cannot be empty.";

	public static InvalidAlias =
		"User name can only have letters and numbers."

	public static AppIssueMessage =
		"Couldn't start application. Check your data connection, then try again.";

	public static NetworkIssueMessage =
		"Couldn't reach the service. Check your data connection.";

	public static SSLIssueMessage =
		"Couldn’t establish SSL connection with Server. Make sure your device's date is set correctly.";

	public static RefreshIssueMessage =
		"Having problems refreshing. Try again in a few minutes.";

	public static CannotReachService =
		"Could't reach the service. Try again in a few minutes.";

	public static LicenseOnly =
		"This user is synchronized with your local Active Directory.Some details can be edited only through your local Active Directory.";

	// Part 2: dom text

	public static LastRefreshPrefix = "Last refreshed";

	public static LastRefreshMinuteMessage = "minute ago";

	public static LastRefreshMinutesMessage = "minutes ago";

	public static LastRefreshMoreThanMinutesMessage = "more than 30 minutes ago";

	public static UserStatusSynced = "Synced with Active Directory";

	public static UserStatusInCloud = "In cloud";

}

class Trace {
	/**
	 *
	 * @param text
	 * @param filename  use for onerror
	 * @param lineno used for oneerror
	 */
	public static errorHandler(text, filename = "", lineno = 0) {
		// only show first error, so do not add listener back
		window.removeEventListener('error', Trace.errorHandler);
		filename = filename || text.filename;
		lineno = lineno || text['lineno'];

		Trace.logExceptions(lineno, filename, text.message, text.error ? text.error.stack : "");
	}


//Check for inserting exception in DB

	/**
	 *
	 * @param status  line number or http return
	 * @param api  local function or server api
	 * @param error exception message or sever error message
	 * @param stack
	 */
	public static logExceptions(status, api, error, stack = "") {
		var message = error + ' @' + api + ":" + status;
		Trace.logError(message + stack);

		if (isDebug())
			Trace.showErrorMessage(message);
		else if (ViewModelBase)
			ViewModelBase.getRoot().asyncSendException(status, api, error, stack); // Send exception to server
	}

	public static notify(msg) {
		if (window.external && "notify" in window.external) // note: do not check for window.external["notify"], otherwise exception is throw
		{
			try {
				window.external["notify"](msg);
			}
			catch (e) {

			}
		}
	}

//Log debugging messages on console
	public static logInfo(s) {
		Trace.notify(s);
		console.log(s);
	}

	public static logVerbose(s) {
		return;
		if (isDebug()) {
			Trace.notify(s);
			console.log(s);
		}
	}

	public static logError(s) {
		Trace.notify("Error:" + s);
		console.error(s);
	}

	public static showErrorMessage(s:string) {
		$(".error").html(s);
	}

	public static clearErrorMessage() {
		//if ($(".error").val())
		Trace.showErrorMessage("");
	}
}

window.addEventListener('error', Trace.errorHandler);

function getStackTrace() {
	var callstack = [];
	try {
		var i;
		i.dont.exist += 0; //doesn't exist- that's the point
	} catch (e) {
		if (e.stack) { // All moden browser supported this
			var lines = e.stack.split('\n');
			var line = lines.shift();
			if (line.indexOf("getStackTrace") < 0)
				lines.shift();

			return lines;
		}
		else {
			var fn = arguments.callee.caller;
			while (fn) {
				callstack.push(getFnName(fn));
				fn = fn.caller;
			}
			return callstack;
		}
	}
}

/* This does not work well on anonymous function*/
function getFnName(fn) {
	var f = typeof fn == 'function';
	var s = f && ((fn.serviceName && ['', fn.serviceName]) || fn.toString().match(/function ([^\(]+)/));
	return (!f && 'not a function') || (s && s[1] || 'anonymous');
}

function selectText(e) {
	//$(e).dblclick(); // not working in ios
	$(e).select(); // not working in ios
	e.setSelectionRange(0, 9999);  // not working in ios
	$(e).css({'color': 'blue'});
}


function ajaxNavigateTo(url, onError) {
	$.ajax(
		{
			type: "GET",
			url: Config.remoteHost + url,
			success: function (response, status, xhr) {
				//$("html").html(response);
				var newDoc = document.open("text/html", "replace");
				newDoc.write(response);
				newDoc.close();
				//console.log(response);
			},
			error: function (xhr, result, error) {
				if (onError)
					onError(xhr, result, error);

			},
			timeout: 5000
		});
}

// avoid function not defined
window["setNative"] = window["onBack"] = window["swiped"] = window["onResume"] = window["onPause"] = function () {
};