/// <reference path="Utilities.ts" />
/// <reference path="Cache.ts" />
/// <reference path="PivotView.ts" />
/// <reference path="jquery.d.ts" />
/// <reference path="knockout.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="knockout.es5.d.ts" />
/// <reference path="LoginView.ts" />

ko.bindingHandlers["stopBinding"] = {
	init: function() {
		return { controlsDescendantBindings: true };
	}
};

class ViewModelBase {
	public static device = new DeviceInfo();
	public static timeInaccurate = false;
	public canRefresh = true;
	public skipHistory = false;
	public ajaxCount = 0;
	public failedAjaxCount = 0;
	public blockUIAjaxCount = 0;
	public searchString;
	public searchPlaceHolder;
	public showToolbar = false;
	public rightToolbarSelected = false;
	public version;
	public cacheLocal:Cache = Cache.getInstance();

	public get preferences() {
		return this.cacheLocal.preferences;
	}

	public get userData() {
		return  this.cacheLocal.userData;
	}

	public get tenantData() {
		return this.cacheLocal.tenantData;
	}

	public menuEnabled = true;
	public static instance: ViewModelBase;
	private _searchCancelClicked = false;
	private _isRoot = false;
	private static _rootInstance:ViewModelBase = null;
	private static _staticInited = false;

	//Get Custom Error Message Based on Error Type
	constructor(isRoot = false) {
		assert(!this.constructor["instance"], "all view are singleton")
		this.constructor["instance"] = this;
		this._isRoot = isRoot;
		if (isRoot) {
			assert(!ViewModelBase._rootInstance);
			ViewModelBase._rootInstance = this;
		}

		this.onReload(true); // reset need to be call before bind
	}

	public setSearchString(val) {
		$('#search').val(val);
		$('#search').blur(); // this will workaround jqm clear button never disappear bug
	}

	public onSearchClearClick() {
		this._searchCancelClicked = true;
	}

	public onSearchBlur() {
		// we should search for everything after user clear search and blur the search box
		if (this._searchCancelClicked && !$('#search').val() && this.searchString)
			this.search("", false)

		this._searchCancelClicked = false;
	}


	public search(searchString:string, blockUI:boolean) {
		assert(!this.searchPlaceHolder);
	}

	public bind() {
		ko.track(this.cacheLocal);
		ko.track(this);

		var id = this.getClassName();
		var _this = this;
		this.loadView(
			this.constructor,
			function () {
				var div = document.getElementById(id);
				assert(div);
				Trace.logInfo("binding to " + id);
				ko.applyBindings(_this, div)
				Trace.logInfo("binded to " + id);
			}
		)
	}

	public loadView(viewClass:Function, callback:Function = null):boolean {
		var viewName = this.getClassName(viewClass);
		var selector = "#" + viewName;

		if ($(selector).children().length == 0) { // need to go to server to load html

			var url = viewName + ".html";
			Trace.logInfo("loading " + url);

			var newCallback = function (response, status, xhr) {
				Trace.logInfo(url + " loaded");
				if (status == "error") { // shall we skip callback?
					Trace.logExceptions(xhr.status, "", "Ajax load pivot " + url + " failed with " + xhr.statusText);
				}
				// For some reason knockout cannot bind the loaded html right away, need to wait for next frame
				// jquery might have some delayed operation also, so this timeout should be big enough
				if (callback)
					setTimeout(callback, 100);
			}

			// Standard XHR won't work on wp local html
			// https://www.scirra.com/forum/please-reconsider-local-file-ajax-issue-in-wp8_t97991
			// workaround is to use activeX and jquery 1.*
			$.ajaxSetup({isLocal: true});
			$(selector).load(url, newCallback);
			$.ajaxSetup({isLocal: false}); // activeX cannot cross domain need to set it back

			return false;
		}
		else {
			if (callback)
				callback();

			return true;
		}

	}

	// This is used to init properties so constructor need to call it
	public onReload(constructor = false) {
		// currently child does not call this
	}

	public getInstance(viewClass:typeof ViewModelBase, reload = false) : ViewModelBase {
		if (viewClass.instance) {
			var instance = viewClass.instance;

			if (reload)
				instance.onReload();

			return instance;
		}
		else
			return new viewClass();
	}

	public static capitalize(s) {
		return s.charAt(0).toUpperCase() + s.slice(1);
	}

	public static unCapitalize(s) {
		return s.charAt(0).toLowerCase() + s.slice(1);
	}

	public getScreenSize():string {
		return $(window).width() + " x " + $(window).height();
	}

	public getClassName(constructor:Function = null):string {
		if (!constructor)
			constructor = this.constructor;

		var funcNameRegex = /function (.{1,})\(/;
		var results = (funcNameRegex).exec(constructor.toString());
		return constructor["name"] || (results && results.length > 1) ? results[1] : "";
	}

	public getTitle():string {
		return "";
	}

	public getSecondaryTitle():string {
		return "";
	}

	public getLeftToolbarText() {
		if (this.searchString)
			return "Filtered by '" + this.searchString + "'"
		else if (this.getItems().length < Config.resultSize)
			return "All"
		else
			return "First " + this.getItems().length;
	}

	public getRightToolbarText() {
		return  "Recent";
	}

	public toggleToolbar(isRight) {
		this.rightToolbarSelected = isRight;
	}

	public getItems() {
		return [];
	}

	public getRightBtn():string {
		if (this.isPrimary())
			return this.getIconClass("setting");
		else
			return "hidden";
	}

	public clickRightBtn() {
		if (this.isPrimary()) {
			this.goTo(SettingsView);
			this.logEvent("Settings");
		}
	}

	public getLeftBtn():string {
		if (!this.isPrimary())
			return "back-btn";
		else
			return "nav-btn";
	}

	public clickLeftBtn() {
		if (!this.isPrimary())
			var backDest = this.goBack();
		else
			$("#nav-panel")["panel"]("toggle");
	}

	public getIconClass(name) {
		return "ms-Icon--" + ViewModelBase.unCapitalize(name).replace(" ", "");
	}

	public static getRoot():ViewModelBase {
		return ViewModelBase._rootInstance;
	}

	public static getLocale() {
		return navigator.language || navigator['userLanguage']
	}

    public static getAppVersion() {
		// Version is in the format of Major.Minor.UI

		var deviceVersions = NativeHelper.getAppVersion().split(".");
		var uiVersions = Config.uiVersion.split(".");

		deviceVersions[2] = uiVersions[2];

		for (var i = 0; i < 2; i++) {
			if (deviceVersions[i])
				uiVersions[i] = deviceVersions[i];
		}

		return uiVersions.join(".") + isDebug();
	}

	public static logCapptainSessionEvent(eventname, params){
		try {
			if (capptain && capptain.agent && capptain.agent.sendSessionEvent) {
				capptain.agent.sendSessionEvent(eventname, params);
			}
		}
		catch (e) {
			Trace.logError(e);
		}
	}

	public static logCapptainEndActivity() {
		try {
			if (capptain && capptain.agent && capptain.agent.endActivity) {
				capptain.agent.endActivity();
			}
		}
		catch (e) {
			Trace.logError(e);
		}
	}

	public static logCapptainStartActivity(activityname) {
		try {
			if (capptain && capptain.agent && capptain.agent.startActivity) {
				capptain.agent.startActivity(activityname);
			}
		}
		catch (e) {
			Trace.logError(e);
		}
	}

	public static logCapptainStartJob(jobname) {
		try {
			if (capptain && capptain.agent && capptain.agent.startJob) {
				capptain.agent.startJob(jobname);
			}
		}
		catch (e) {
			Trace.logError(e);
		}
	}

	public static logCapptainEndJob(jobname) {
		try {
			if (capptain && capptain.agent && capptain.agent.endJob) {
				capptain.agent.endJob(jobname);
			}
		}
		catch (e) {
			Trace.logError(e);
		}
	}

	public static logCapptainAppInfo(params) {
		try {
			if (capptain && capptain.agent && capptain.agent.sendAppInfo) {
				capptain.agent.sendAppInfo(params);
			}
		}
		catch (e) {
			Trace.logError(e);
		}
	}

	public static logCapptainError(errMsg, params) {
		try {
			if (capptain && capptain.agent && capptain.agent.sendError) {
				capptain.agent.sendError(errMsg, params);
			}
		}
		catch (e) {
			Trace.logError(e);
		}
	}

	public isWebView():boolean {
		return !!NativeHelper.getAppVersion();
	}

	public logEvent(target, action = "Click", para = null) {
		assert(target);
		Trace.logInfo(target + " " + action);
		target = ViewModelBase.capitalize(target);
		var eventName = action + target;

		if (FlurryAgent && FlurryAgent.logEvent) {
			var parameters = { // all param should be string
				AOBO: this.isAOBO()
			};
			FlurryAgent.logEvent(eventName, parameters);
		}

		if (action == "Click") {
			ViewModelBase.logCapptainSessionEvent(eventName, {
				AOBO: this.isAOBO()
			});
		}
	}

	public pageAnimation() {
		return Config.pageAnimation;
	}

	public static getDeviceOS() {
		if (ViewModelBase.device.winTablet)
			return "winphoneT";
		else if (ViewModelBase.device.winPhone)
			return "winphone";
		else if (ViewModelBase.device.androidTablet)
			return "androidT";
		else if (ViewModelBase.device.android)
			return "android";
		else if (ViewModelBase.device.iPad)
			return "ipad";
		else if (ViewModelBase.device.iPhone)
			return "iphone";
		else
			return "";
	}

	public asyncRequest(serviceURL, api, serviceParams, onSuccess = null, onError = null /*depreciated*/, blockUI = false) {

		ViewModelBase.logCapptainStartJob(api);

		if (this != ViewModelBase.getRoot()) {
			Trace.clearErrorMessage();
		}

		if (serviceParams) {
			if (!serviceParams.resultSize) // for now add resultSize to all requests and overfetch a little more
				serviceParams.resultSize = Config.resultSize + 10;

			var postData = JSON.stringify(serviceParams);
			var action = "POST";

			// Clear sensitive information
			if (serviceParams.lastCookie)
				serviceParams.lastCookie = serviceParams.lastCookie.substring(0, 10);

			delete serviceParams.password;
		}
		else {
			action = "GET";
		}

		//Trace.clearErrorMessage();
		Trace.logInfo(serviceURL + api + JSON.stringify(serviceParams));

		this.ajaxCount++;

		if (blockUI)
			this.blockUIAjaxCount++

		var _this = this;

		var request = {
			type: action,
			url: serviceURL + api,
			data: postData,
			contentType: 'application/json; charset=utf-8',
			dataType: "json",
			success: function (response, status, xhr) {
				var data = response;

				// Fetch value if keyValue pair is returned
				if (serviceParams) {
					var domains = serviceParams.domains || serviceParams.tenantDomains;
					if (domains && domains.length == 1 && domains[0]) {
						if (response.length == 1 && response[0].Key == domains[0]) {
							data = response[0].Value;
						}
					}
				}

				if (onSuccess) {
					onSuccess.call(_this, data);
				}
			},
			timeout: isDebug() ? 120000 : 25000,
			error: function (xhr:XMLHttpRequest, result, error) {
				var status = xhr.status;

				// Workaround for IE10 https://connect.microsoft.com/IE/feedback/details/773478/ie-10-on-win8-does-not-assign-the-correct-httpstatus-to-xmlhttprequest-when-the-result-is-401
				// There is no better workaround since activeX request cannot cross domain
				var isIE10 = navigator.userAgent.indexOf("MSIE 10") > -1;
				if (xhr.status === 0 && error == "error" && isIE10 && !ViewModelBase.timeInaccurate && NativeHelper.isNetworkAvailable()) {
					status = 401;
					error = "IE10";
				}

				_this.failedAjaxCount++;

				if (onError) {
					var handled = onError.call(_this, status, error, api, blockUI)
				}
				else {
					handled = _this.onAjaxError(status, xhr.statusText, error, api, blockUI);
				}

				var errorString = "statusText: " + xhr.statusText + " error:" + error + " from:" + _this.getClassName();

				if (!handled && api != Config.sendExceptiontag)
					Trace.logExceptions(status, serviceURL + api, errorString , postData);
				else
					Trace.logInfo(serviceURL + api + ":" + status + " "  + errorString + " " + postData);
			},
			complete: function (jqXHR, status) {
				_this.cacheLocal.save();
				_this.ajaxCount--;
				if (blockUI)
					_this.blockUIAjaxCount--;
			}
		}

		$.ajax(request);

		ViewModelBase.logCapptainEndJob(api);

		this.logEvent(api, "Ajax", serviceParams);
	}

	// return if error has been handled
	public onAjaxError(statusCode, statusDescription, error, api, blockUI):boolean {
		if (api == Config.timeTag)
		{
			this.cacheLocal.isNetworkAvailable = false;
			return true;
		}
		else if (api == Config.sendExceptiontag){
			return true;
		}
		else if (statusCode === 0) {
			if (!(this instanceof LoginView) && !this.userData.cookie)
				return true; // when signout is called, ajax may be cancelled. no need to show error message in this case
			else if (ViewModelBase.timeInaccurate)
				Trace.showErrorMessage(Strings.SSLIssueMessage);
			else if (this.cacheLocal.isNetworkAvailable)
				Trace.showErrorMessage(Strings.CannotReachService);
			else
				Trace.showErrorMessage(Strings.NetworkIssueMessage); // Chrome may suspend connection

			return true;
		}
		else if (statusCode === 401 && !(this instanceof LoginView) && statusDescription.indexOf("The registration cookie expired") >= 0) {
			this.signOut(true);
		}
		else if (statusCode === 400 && !(this instanceof LoginView) && statusDescription.indexOf("lastCookie") >= 0) {
			this.signOut(true);
		}
		else if (statusDescription.indexOf("AccessDeniedException") >= 0 && blockUI) { // usually 400
			Trace.showErrorMessage(Strings.AccessDenied);
		}
		else if (statusCode === 503) {
			Trace.showErrorMessage("Max requests per minute reached. Please try again later.")
		}
		else if (statusCode === 500 && blockUI) {
			Trace.showErrorMessage(Strings.InternalServerError);
			return false;
		}
		else if (error === "timeout") {
			Trace.showErrorMessage(Strings.CannotReachService);
		}
		else {
			return false;
		}

		return true;
	}

	/**
	 *
	 * @param status  line number or http return
	 * @param api  local function or server api
	 * @param error exception message or sever error message
	 * @param stack
	 */
	public asyncSendException(status, api, error, stack = "") {
		var serviceParams = {
			userName: this.preferences.userName,
			apiName: api + ":" + status, // todo: add status as separate param
			appVersion: ViewModelBase.getAppVersion(),
			operatingSystem: navigator.userAgent,
			exeptionDetails: error,
			callStacks: stack,
			deviceId: ""
		}

		this.asyncRequest(
			Config.serviceURL,
			Config.sendExceptiontag,
			serviceParams,
			null);
	}

	// hearbeat to check network
	public asyncCheckTime() {
		// ie cannot call http from https
		if (window.location.href.indexOf("https") > -1) {
			this.cacheLocal.isNetworkAvailable = true;
			ViewModelBase.timeInaccurate = false;
			return;
		}

		var _this = this;
		var success = function (response) {
			var serverTime = new Date(response[0]);
			var localTime = new Date();
			_this.cacheLocal.isNetworkAvailable = true;

			if (Math.abs(serverTime.getTime() - localTime.getTime()) > 48 * 60 * 60 * 1000) // 2 day
				ViewModelBase.timeInaccurate = true;
			else
				ViewModelBase.timeInaccurate = false;
		};

		this.asyncRequest(Config.timeURL, Config.timeTag, "", success);

	}

	/**
	 *
	 * @returns  empty string to use default back button behavior otherwise the new url
	 */
	public goBack():string {
		if (window.history.length >= 2) { // Do not go back to login page
			Trace.clearErrorMessage();
			window.history.back();
			return window.location.href;
		}
		else {
			return ""; // quite app
		}
	}

	/// Method to navigate to Login page
	public navigateTo(page) {
		this.ajaxCount = Number.MAX_VALUE; // Continue to show progress bar until new page is loaded.
		this.cacheLocal.save();

		if (Config.useAjaxNavigation)
			ajaxNavigateTo(page, null);
		else if (window.location.href.indexOf("Login.html") >= 0)
			window.location.replace(page); // avoid go back to login
		else
			window.location.href = page;

		if (FlurryAgent && FlurryAgent.endSession) {
			FlurryAgent.endSession();
		}

		if (page == "Login.html") {
			ViewModelBase.logCapptainEndActivity();
			ViewModelBase.logCapptainStartActivity("SignOut");
			ViewModelBase.logCapptainEndActivity();
		}
	}

	public goTo(pivotClass:typeof ViewModelBase) {
		this.goToPivot(this.getClassName(pivotClass));
	}

	public goToPivot(pivot:string) {
		ViewModelBase.getRoot().goToPivot(pivot)
	}

	public static getDate(jsonDate:string):Date {
		return new Date(parseInt(jsonDate.substring(6)));
	}

	public static formatHtml(text:string):string {
		//return text.replace("\r\n", "<br><br>");

		return  text.replace(/\r\n/g, '<br>').replace(/[\r\n]/g, '<br>');
	}

	//Get Formatted Date
	public static getFormattedDate(jsonDate:string, dateOnly = true):string {
		if (!jsonDate)
			return "";

		var date = ViewModelBase.getDate(jsonDate);

		if (dateOnly)
			return ViewModelBase.getDateString(date);
		else
			return ViewModelBase.getDateString(date) + " " + ViewModelBase.getTimeString(date);
	}

	public static getTimeString(date:Date):string {
		if (Config.localizeTime)
			return date.toLocaleTimeString();
		else {

			var hours = date.getHours();
			var minutes = date.getMinutes();

			return (hours < 10 ? "0" : "") + hours + ":" + (minutes < 10 ? "0" : "") + minutes;
		}
	}

	public static getDateString(date:Date):string {
		var formats = {"ar-SA": "dd/MM/yy", "bg-BG": "dd.M.yyyy", "ca-ES": "dd/MM/yyyy", "zh-TW": "yyyy/M/d", "cs-CZ": "d.M.yyyy", "da-DK": "dd-MM-yyyy", "de-DE": "dd.MM.yyyy", "el-GR": "d/M/yyyy", "en-US": "M/d/yyyy", "fi-FI": "d.M.yyyy", "fr-FR": "dd/MM/yyyy", "he-IL": "dd/MM/yyyy", "hu-HU": "yyyy. MM. dd.", "is-IS": "d.M.yyyy", "it-IT": "dd/MM/yyyy", "ja-JP": "yyyy/MM/dd", "ko-KR": "yyyy-MM-dd", "nl-NL": "d-M-yyyy", "nb-NO": "dd.MM.yyyy", "pl-PL": "yyyy-MM-dd", "pt-BR": "d/M/yyyy", "ro-RO": "dd.MM.yyyy", "ru-RU": "dd.MM.yyyy", "hr-HR": "d.M.yyyy", "sk-SK": "d. M. yyyy", "sq-AL": "yyyy-MM-dd", "sv-SE": "yyyy-MM-dd", "th-TH": "d/M/yyyy", "tr-TR": "dd.MM.yyyy", "ur-PK": "dd/MM/yyyy", "id-ID": "dd/MM/yyyy", "uk-UA": "dd.MM.yyyy", "be-BY": "dd.MM.yyyy", "sl-SI": "d.M.yyyy", "et-EE": "d.MM.yyyy", "lv-LV": "yyyy.MM.dd.", "lt-LT": "yyyy.MM.dd", "fa-IR": "MM/dd/yyyy", "vi-VN": "dd/MM/yyyy", "hy-AM": "dd.MM.yyyy", "az-Latn-AZ": "dd.MM.yyyy", "eu-ES": "yyyy/MM/dd", "mk-MK": "dd.MM.yyyy", "af-ZA": "yyyy/MM/dd", "ka-GE": "dd.MM.yyyy", "fo-FO": "dd-MM-yyyy", "hi-IN": "dd-MM-yyyy", "ms-MY": "dd/MM/yyyy", "kk-KZ": "dd.MM.yyyy", "ky-KG": "dd.MM.yy", "sw-KE": "M/d/yyyy", "uz-Latn-UZ": "dd/MM yyyy", "tt-RU": "dd.MM.yyyy", "pa-IN": "dd-MM-yy", "gu-IN": "dd-MM-yy", "ta-IN": "dd-MM-yyyy", "te-IN": "dd-MM-yy", "kn-IN": "dd-MM-yy", "mr-IN": "dd-MM-yyyy", "sa-IN": "dd-MM-yyyy", "mn-MN": "yy.MM.dd", "gl-ES": "dd/MM/yy", "kok-IN": "dd-MM-yyyy", "syr-SY": "dd/MM/yyyy", "dv-MV": "dd/MM/yy", "ar-IQ": "dd/MM/yyyy", "zh-CN": "yyyy/M/d", "de-CH": "dd.MM.yyyy", "en-GB": "dd/MM/yyyy", "es-MX": "dd/MM/yyyy", "fr-BE": "d/MM/yyyy", "it-CH": "dd.MM.yyyy", "nl-BE": "d/MM/yyyy", "nn-NO": "dd.MM.yyyy", "pt-PT": "dd-MM-yyyy", "sr-Latn-CS": "d.M.yyyy", "sv-FI": "d.M.yyyy", "az-Cyrl-AZ": "dd.MM.yyyy", "ms-BN": "dd/MM/yyyy", "uz-Cyrl-UZ": "dd.MM.yyyy", "ar-EG": "dd/MM/yyyy", "zh-HK": "d/M/yyyy", "de-AT": "dd.MM.yyyy", "en-AU": "d/MM/yyyy", "es-ES": "dd/MM/yyyy", "fr-CA": "yyyy-MM-dd", "sr-Cyrl-CS": "d.M.yyyy", "ar-LY": "dd/MM/yyyy", "zh-SG": "d/M/yyyy", "de-LU": "dd.MM.yyyy", "en-CA": "dd/MM/yyyy", "es-GT": "dd/MM/yyyy", "fr-CH": "dd.MM.yyyy", "ar-DZ": "dd-MM-yyyy", "zh-MO": "d/M/yyyy", "de-LI": "dd.MM.yyyy", "en-NZ": "d/MM/yyyy", "es-CR": "dd/MM/yyyy", "fr-LU": "dd/MM/yyyy", "ar-MA": "dd-MM-yyyy", "en-IE": "dd/MM/yyyy", "es-PA": "MM/dd/yyyy", "fr-MC": "dd/MM/yyyy", "ar-TN": "dd-MM-yyyy", "en-ZA": "yyyy/MM/dd", "es-DO": "dd/MM/yyyy", "ar-OM": "dd/MM/yyyy", "en-JM": "dd/MM/yyyy", "es-VE": "dd/MM/yyyy", "ar-YE": "dd/MM/yyyy", "en-029": "MM/dd/yyyy", "es-CO": "dd/MM/yyyy", "ar-SY": "dd/MM/yyyy", "en-BZ": "dd/MM/yyyy", "es-PE": "dd/MM/yyyy", "ar-JO": "dd/MM/yyyy", "en-TT": "dd/MM/yyyy", "es-AR": "dd/MM/yyyy", "ar-LB": "dd/MM/yyyy", "en-ZW": "M/d/yyyy", "es-EC": "dd/MM/yyyy", "ar-KW": "dd/MM/yyyy", "en-PH": "M/d/yyyy", "es-CL": "dd-MM-yyyy", "ar-AE": "dd/MM/yyyy", "es-UY": "dd/MM/yyyy", "ar-BH": "dd/MM/yyyy", "es-PY": "dd/MM/yyyy", "ar-QA": "dd/MM/yyyy", "es-BO": "dd/MM/yyyy", "es-SV": "dd/MM/yyyy", "es-HN": "dd/MM/yyyy", "es-NI": "dd/MM/yyyy", "es-PR": "dd/MM/yyyy", "am-ET": "d/M/yyyy", "tzm-Latn-DZ": "dd-MM-yyyy", "iu-Latn-CA": "d/MM/yyyy", "sma-NO": "dd.MM.yyyy", "mn-Mong-CN": "yyyy/M/d", "gd-GB": "dd/MM/yyyy", "en-MY": "d/M/yyyy", "prs-AF": "dd/MM/yy", "bn-BD": "dd-MM-yy", "wo-SN": "dd/MM/yyyy", "rw-RW": "M/d/yyyy", "qut-GT": "dd/MM/yyyy", "sah-RU": "MM.dd.yyyy", "gsw-FR": "dd/MM/yyyy", "co-FR": "dd/MM/yyyy", "oc-FR": "dd/MM/yyyy", "mi-NZ": "dd/MM/yyyy", "ga-IE": "dd/MM/yyyy", "se-SE": "yyyy-MM-dd", "br-FR": "dd/MM/yyyy", "smn-FI": "d.M.yyyy", "moh-CA": "M/d/yyyy", "arn-CL": "dd-MM-yyyy", "ii-CN": "yyyy/M/d", "dsb-DE": "d. M. yyyy", "ig-NG": "d/M/yyyy", "kl-GL": "dd-MM-yyyy", "lb-LU": "dd/MM/yyyy", "ba-RU": "dd.MM.yy", "nso-ZA": "yyyy/MM/dd", "quz-BO": "dd/MM/yyyy", "yo-NG": "d/M/yyyy", "ha-Latn-NG": "d/M/yyyy", "fil-PH": "M/d/yyyy", "ps-AF": "dd/MM/yy", "fy-NL": "d-M-yyyy", "ne-NP": "M/d/yyyy", "se-NO": "dd.MM.yyyy", "iu-Cans-CA": "d/M/yyyy", "sr-Latn-RS": "d.M.yyyy", "si-LK": "yyyy-MM-dd", "sr-Cyrl-RS": "d.M.yyyy", "lo-LA": "dd/MM/yyyy", "km-KH": "yyyy-MM-dd", "cy-GB": "dd/MM/yyyy", "bo-CN": "yyyy/M/d", "sms-FI": "d.M.yyyy", "as-IN": "dd-MM-yyyy", "ml-IN": "dd-MM-yy", "en-IN": "dd-MM-yyyy", "or-IN": "dd-MM-yy", "bn-IN": "dd-MM-yy", "tk-TM": "dd.MM.yy", "bs-Latn-BA": "d.M.yyyy", "mt-MT": "dd/MM/yyyy", "sr-Cyrl-ME": "d.M.yyyy", "se-FI": "d.M.yyyy", "zu-ZA": "yyyy/MM/dd", "xh-ZA": "yyyy/MM/dd", "tn-ZA": "yyyy/MM/dd", "hsb-DE": "d. M. yyyy", "bs-Cyrl-BA": "d.M.yyyy", "tg-Cyrl-TJ": "dd.MM.yy", "sr-Latn-BA": "d.M.yyyy", "smj-NO": "dd.MM.yyyy", "rm-CH": "dd/MM/yyyy", "smj-SE": "yyyy-MM-dd", "quz-EC": "dd/MM/yyyy", "quz-PE": "dd/MM/yyyy", "hr-BA": "d.M.yyyy.", "sr-Latn-ME": "d.M.yyyy", "sma-SE": "yyyy-MM-dd", "en-SG": "d/M/yyyy", "ug-CN": "yyyy-M-d", "sr-Cyrl-BA": "d.M.yyyy", "es-US": "M/d/yyyy"};

		var l = ViewModelBase.getLocale();
		var y = date.getFullYear();
		var m = date.getMonth() + 1;
		var d = date.getDate();
		var f = (formats[l] && Config.localizeDate) ? formats[l] : "MM/dd/yyyy";

		function double(s) {
			s = '' + s;
			return s.length > 1 ? s : '0' + s;
		}

		f = f.replace(/yyyy/, y);
		f = f.replace(/yy/, String(y).substr(2));
		f = f.replace(/MM/, double(m));
		f = f.replace(/M/, m);
		f = f.replace(/dd/, double(d));
		f = f.replace(/d/, d);
		return f;
	}

	public static clone(object: any) {
		return JSON.parse(JSON.stringify(object));
	}

	//Check if  Network if available or not
	public isNetworkAvailable() {
		try {
			return NativeHelper.isNetworkAvailable();
		} catch (error) {
			return true;
		}
	}

	public isPrimary() {
		return false;
	}

	public isAOBO():boolean {
		return this.tenantData.isAOBO;
	}

	public isPartner() {
		return this.userData.userRole == UserRole.MicrosoftPartner && this.userData.partnerRole != PartnerRole.None;
	}

	public isPartnerNotAOBO() {

		if (Config.allowAggregatedHealth)
			return this.isPartner() && !this.isAOBO();
		else
			return false;
	}

	public getCurrentRefreshMinutes():number {
		var now = (new Date()).getTime();
		return (now - this.tenantData.lastEventRefreshMilliSec) / 60000 | 0;
	}

	public signOut(cookieExpired = false) {
		if (cookieExpired) {
			this.userData.cookieExpired = true;
			Trace.showErrorMessage(Strings.CookieExpiredMessage);
		}
		this.userData.cookie = "";
		this.navigateTo("Login.html");
	}

	public assert(condition, message = "") {
		message = message || "assert error";

		if (!condition) {
			throw message;
		}
	}

	public static scrollTop() {
		// workaround for navbar flickering after keyboard disappear
		window.scrollTo(document.body.scrollLeft, document.body.scrollTop);
	}

	// Workaround if UI not updated by binding. Not used for now.
	public static valueHasChanged(o, p) {
		var tmp = o[p];

		if (o[p] && o[p].length)
			o[p] = [];
		else
			o[p] = null;

		o[p] = tmp;
	}
}

module capptain {
	export var appId = 'opa000000';
	export var appVersionName = ViewModelBase.getAppVersion();
	export var appVersionCode = 1;
	export var agent: Agent;

	export interface Agent {
		startActivity: Function;
		endActivity: Function;
		pause: Function;
		sendSessionEvent: Function;
		sendSessionError: Function;
		sendError: Function;
		endJob: Function;
		startJob: Function;
		sendJobError: Function;
		sendJobEvent: Function;
		sendCrash: Function;
		sendEvent: Function; // parameter: JSON object
		sendAppInfo: Function; // parameter: JSON object
	}
}
